@@ -1,23 +1,22 @@
 /*
  * ngIRCd -- The Next Generation IRC Daemon
- * Copyright (c)2001-2005 by Alexander Barton (alex@barton.de)
+ * Copyright (c)2001-2010 Alexander Barton (alex@barton.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  * Please read the file COPYING, README and AUTHORS for more information.
- *
- * Client management.
  */
 
-
 #define __client_c__
 
-
 #include "portab.h"
 
-static char UNUSED id[] = "$Id: client.c,v 1.98 2008/04/04 19:30:01 fw Exp $";
+/**
+ * @file
+ * Client management.
+ */
 
 #include "imp.h"
 #include <assert.h>
@@ -37,8 +36,8 @@ static char UNUSED id[] = "$Id: client.c,v 1.98 2008/04/04 19:30:01 fw Exp $";
 #include <imp.h>
 #include "ngircd.h"
 #include "channel.h"
-#include "resolve.h"
 #include "conf.h"
+#include "conn-func.h"
 #include "hash.h"
 #include "irc-write.h"
 #include "log.h"
@@ -46,12 +45,9 @@ static char UNUSED id[] = "$Id: client.c,v 1.98 2008/04/04 19:30:01 fw Exp $";
 
 #include <exp.h>
 
-
 #define GETID_LEN (CLIENT_NICK_LEN-1) + 1 + (CLIENT_USER_LEN-1) + 1 + (CLIENT_HOST_LEN-1) + 1
 
-
 static CLIENT *This_Server, *My_Clients;
-static char GetID_Buffer[GETID_LEN];
 
 static WHOWAS My_Whowas[MAX_WHOWAS];
 static int Last_Whowas = -1;
@@ -66,9 +62,16 @@ static void Generate_MyToken PARAMS(( CLIENT *Client ));
 static void Adjust_Counters PARAMS(( CLIENT *Client ));
 
 static CLIENT *Init_New_Client PARAMS((CONN_ID Idx, CLIENT *Introducer,
- CLIENT *TopServer, int Type, char *ID, char *User, char *Hostname,
- char *Info, int Hops, int Token, char *Modes, bool Idented));
+				       CLIENT *TopServer, int Type, const char *ID,
+				       const char *User, const char *Hostname, const char *Info,
+				       int Hops, int Token, const char *Modes,
+				       bool Idented));
 
+static void Destroy_UserOrService PARAMS((CLIENT *Client,const char *Txt, const char *FwdMsg,
+					bool SendQuit));
+
+static void cb_introduceClient PARAMS((CLIENT *Client, CLIENT *Prefix,
+				       void *i));
 
 GLOBAL void
 Client_Init( void )
@@ -92,7 +95,7 @@ Client_Init( void )
 	This_Server->hops = 0;
 
 	gethostname( This_Server->host, CLIENT_HOST_LEN );
-	if (!Conf_NoDNS) {
+	if (Conf_DNS) {
 		h = gethostbyname( This_Server->host );
 		if (h) strlcpy(This_Server->host, h->h_name, sizeof(This_Server->host));
 	}
@@ -139,7 +142,7 @@ Client_ThisServer( void )
  * @return New CLIENT structure.
  */
 GLOBAL CLIENT *
-Client_NewLocal(CONN_ID Idx, char *Hostname, int Type, bool Idented)
+Client_NewLocal(CONN_ID Idx, const char *Hostname, int Type, bool Idented)
 {
 	return Init_New_Client(Idx, This_Server, NULL, Type, NULL, NULL,
 		Hostname, NULL, 0, 0, NULL, Idented);
@@ -151,8 +154,8 @@ Client_NewLocal(CONN_ID Idx, char *Hostname, int Type, bool Idented)
  * @return New CLIENT structure.
  */
 GLOBAL CLIENT *
-Client_NewRemoteServer(CLIENT *Introducer, char *Hostname, CLIENT *TopServer,
- int Hops, int Token, char *Info, bool Idented)
+Client_NewRemoteServer(CLIENT *Introducer, const char *Hostname, CLIENT *TopServer,
+ int Hops, int Token, const char *Info, bool Idented)
 {
 	return Init_New_Client(NONE, Introducer, TopServer, CLIENT_SERVER,
 		Hostname, NULL, Hostname, Info, Hops, Token, NULL, Idented);
@@ -164,8 +167,8 @@ Client_NewRemoteServer(CLIENT *Introducer, char *Hostname, CLIENT *TopServer,
  * @return New CLIENT structure.
  */
 GLOBAL CLIENT *
-Client_NewRemoteUser(CLIENT *Introducer, char *Nick, int Hops, char *User,
- char *Hostname, int Token, char *Modes, char *Info, bool Idented)
+Client_NewRemoteUser(CLIENT *Introducer, const char *Nick, int Hops, const char *User,
+ const char *Hostname, int Token, const char *Modes, const char *Info, bool Idented)
 {
 	return Init_New_Client(NONE, Introducer, NULL, CLIENT_USER, Nick,
 		User, Hostname, Info, Hops, Token, Modes, Idented);
@@ -179,54 +182,60 @@ Client_NewRemoteUser(CLIENT *Introducer, char *Nick, int Hops, char *User,
  */
 static CLIENT *
 Init_New_Client(CONN_ID Idx, CLIENT *Introducer, CLIENT *TopServer,
- int Type, char *ID, char *User, char *Hostname, char *Info, int Hops,
- int Token, char *Modes, bool Idented)
+  int Type, const char *ID, const char *User, const char *Hostname,
+  const char *Info, int Hops, int Token, const char *Modes, bool Idented)
 {
 	CLIENT *client;
 
-	assert( Idx >= NONE );
-	assert( Introducer != NULL );
-	assert( Hostname != NULL );
+	assert(Idx >= NONE);
+	assert(Introducer != NULL);
 
-	client = New_Client_Struct( );
-	if( ! client ) return NULL;
+	client = New_Client_Struct();
+	if (!client)
+		return NULL;
 
-	/* Initialisieren */
 	client->starttime = time(NULL);
 	client->conn_id = Idx;
 	client->introducer = Introducer;
 	client->topserver = TopServer;
 	client->type = Type;
-	if( ID ) Client_SetID( client, ID );
-	if( User ) Client_SetUser( client, User, Idented );
-	if( Hostname ) Client_SetHostname( client, Hostname );
-	if( Info ) Client_SetInfo( client, Info );
+	if (ID)
+		Client_SetID(client, ID);
+	if (User) {
+		Client_SetUser(client, User, Idented);
+		Client_SetOrigUser(client, User);
+	}
+	if (Hostname)
+		Client_SetHostname(client, Hostname);
+	if (Info)
+		Client_SetInfo(client, Info);
 	client->hops = Hops;
 	client->token = Token;
-	if( Modes ) Client_SetModes( client, Modes );
-	if( Type == CLIENT_SERVER ) Generate_MyToken( client );
+	if (Modes)
+		Client_SetModes(client, Modes);
+	if (Type == CLIENT_SERVER)
+		Generate_MyToken(client);
 
-	if( strchr( client->modes, 'a' ))
-		strlcpy( client->away, DEFAULT_AWAY_MSG, sizeof( client->away ));
+	if (strchr(client->modes, 'a'))
+		strlcpy(client->away, DEFAULT_AWAY_MSG, sizeof(client->away));
 
-	/* Verketten */
 	client->next = (POINTER *)My_Clients;
 	My_Clients = client;
 
-	/* Adjust counters */
-	Adjust_Counters( client );
+	Adjust_Counters(client);
 
 	return client;
 } /* Init_New_Client */
 
 
 GLOBAL void
-Client_Destroy( CLIENT *Client, char *LogMsg, char *FwdMsg, bool SendQuit )
+Client_Destroy( CLIENT *Client, const char *LogMsg, const char *FwdMsg, bool SendQuit )
 {
-	/* Client entfernen. */
+	/* remove a client */
 	
 	CLIENT *last, *c;
-	char msg[LINE_LEN], *txt;
+	char msg[LINE_LEN];
+	const char *txt;
 
 	assert( Client != NULL );
 
@@ -234,7 +243,7 @@ Client_Destroy( CLIENT *Client, char *LogMsg, char *FwdMsg, bool SendQuit )
 	else txt = FwdMsg;
 	if( ! txt ) txt = "Reason unknown.";
 
-	/* Netz-Split-Nachricht vorbereiten (noch nicht optimal) */
+	/* netsplit message */
 	if( Client->type == CLIENT_SERVER ) {
 		strlcpy(msg, This_Server->id, sizeof (msg));
 		strlcat(msg, " ", sizeof (msg));
@@ -247,8 +256,16 @@ Client_Destroy( CLIENT *Client, char *LogMsg, char *FwdMsg, bool SendQuit )
 	{
 		if(( Client->type == CLIENT_SERVER ) && ( c->introducer == Client ) && ( c != Client ))
 		{
-			/* der Client, der geloescht wird ist ein Server. Der Client, den wir gerade
-			 * pruefen, ist ein Child von diesem und muss daher auch entfernt werden */
+			/*
+			 * The client that is about to be removed is a server,
+			 * the client we are checking right now is a child of that
+			 * server and thus has to be removed, too.
+			 *
+			 * Call Client_Destroy() recursively with the server as the
+			 * new "object to be removed". This starts the cycle again, until
+			 * all servers that are linked via the original server have been
+			 * removed.
+			 */
 			Client_Destroy( c, NULL, msg, false );
 			last = NULL;
 			c = My_Clients;
@@ -256,45 +273,12 @@ Client_Destroy( CLIENT *Client, char *LogMsg, char *FwdMsg, bool SendQuit )
 		}
 		if( c == Client )
 		{
-			/* Wir haben den Client gefunden: entfernen */
+			/* found  the client: remove it */
 			if( last ) last->next = c->next;
 			else My_Clients = (CLIENT *)c->next;
 
-			if( c->type == CLIENT_USER )
-			{
-				if( c->conn_id != NONE )
-				{
-					/* Ein lokaler User */
-					Log( LOG_NOTICE, "User \"%s\" unregistered (connection %d): %s", Client_Mask( c ), c->conn_id, txt );
-
-					if( SendQuit )
-					{
-						/* Alle andere Server informieren! */
-						if( FwdMsg ) IRC_WriteStrServersPrefix( NULL, c, "QUIT :%s", FwdMsg );
-						else IRC_WriteStrServersPrefix( NULL, c, "QUIT :" );
-					}
-				}
-				else
-				{
-					/* Remote User */
-					Log( LOG_DEBUG, "User \"%s\" unregistered: %s", Client_Mask( c ), txt );
-
-					if( SendQuit )
-					{
-						/* Andere Server informieren, ausser denen, die "in
-						 * Richtung dem liegen", auf dem der User registriert
-						 * ist. Von denen haben wir das QUIT ja wohl bekommen. */
-						if( FwdMsg ) IRC_WriteStrServersPrefix( Client_NextHop( c ), c, "QUIT :%s", FwdMsg );
-						else IRC_WriteStrServersPrefix( Client_NextHop( c ), c, "QUIT :" );
-					}
-				}
-
-				/* Unregister client from channels */
-				Channel_Quit( c, FwdMsg ? FwdMsg : c->id );
-				
-				/* Register client in My_Whowas structure */
-				Client_RegisterWhowas( c );
-			}
+			if(c->type == CLIENT_USER || c->type == CLIENT_SERVICE)
+				Destroy_UserOrService(c, txt, FwdMsg, SendQuit);
 			else if( c->type == CLIENT_SERVER )
 			{
 				if( c != This_Server )
@@ -303,7 +287,7 @@ Client_Destroy( CLIENT *Client, char *LogMsg, char *FwdMsg, bool SendQuit )
 					else Log( LOG_NOTICE|LOG_snotice, "Server \"%s\" unregistered: %s", c->id, txt );
 				}
 
-				/* andere Server informieren */
+				/* inform other servers */
 				if( ! NGIRCd_SignalQuit )
 				{
 					if( FwdMsg ) IRC_WriteStrServersPrefix( Client_NextHop( c ), c, "SQUIT %s :%s", c->id, FwdMsg );
@@ -331,100 +315,139 @@ Client_Destroy( CLIENT *Client, char *LogMsg, char *FwdMsg, bool SendQuit )
 } /* Client_Destroy */
 
 
+/**
+ * Set client hostname.
+ *
+ * If global hostname cloaking is in effect, don't set the real hostname
+ * but the configured one.
+ *
+ * @param Client The client of which the hostname should be set.
+ * @param Hostname The new hostname.
+ */
 GLOBAL void
-Client_SetHostname( CLIENT *Client, char *Hostname )
+Client_SetHostname( CLIENT *Client, const char *Hostname )
 {
-	/* Hostname eines Clients setzen */
+	assert(Client != NULL);
+	assert(Hostname != NULL);
 
-	assert( Client != NULL );
-	assert( Hostname != NULL );
-
-	strlcpy( Client->host, Hostname, sizeof( Client->host ));
+	if (strlen(Conf_CloakHost)) {
+		LogDebug("Updating hostname of \"%s\": \"%s\" -> \"%s\"",
+			 Client_ID(Client), Client->host, Conf_CloakHost);
+		strlcpy(Client->host, Conf_CloakHost, sizeof(Client->host));
+	} else {
+		LogDebug("Updating hostname of \"%s\": \"%s\" -> \"%s\"",
+			 Client_ID(Client), Client->host, Hostname);
+		strlcpy(Client->host, Hostname, sizeof(Client->host));
+	}
 } /* Client_SetHostname */
 
 
 GLOBAL void
-Client_SetID( CLIENT *Client, char *ID )
+Client_SetID( CLIENT *Client, const char *ID )
 {
-	/* Hostname eines Clients setzen, Hash-Wert berechnen */
-
 	assert( Client != NULL );
 	assert( ID != NULL );
 	
 	strlcpy( Client->id, ID, sizeof( Client->id ));
 
+	if (Conf_CloakUserToNick) {
+		strlcpy( Client->user, ID, sizeof( Client->user ));
+		strlcpy( Client->info, ID, sizeof( Client->info ));
+	}
+
 	/* Hash */
 	Client->hash = Hash( Client->id );
 } /* Client_SetID */
 
 
 GLOBAL void
-Client_SetUser( CLIENT *Client, char *User, bool Idented )
+Client_SetUser( CLIENT *Client, const char *User, bool Idented )
 {
-	/* Username eines Clients setzen */
+	/* set clients username */
 
 	assert( Client != NULL );
 	assert( User != NULL );
 
-	if( Idented ) strlcpy( Client->user, User, sizeof( Client->user ));
-	else
-	{
+	if (Conf_CloakUserToNick) {
+		strlcpy(Client->user, Client->id, sizeof(Client->user));
+	} else if (Idented) {
+		strlcpy(Client->user, User, sizeof(Client->user));
+	} else {
 		Client->user[0] = '~';
-		strlcpy( Client->user + 1, User, sizeof( Client->user ) - 1 );
+		strlcpy(Client->user + 1, User, sizeof(Client->user) - 1);
 	}
 } /* Client_SetUser */
 
 
+/**
+ * Set "original" user name of a client.
+ * This function saves the "original" user name, the user name specified by
+ * the peer using the USER command, into the CLIENT structure. This user
+ * name may be used for authentication, for example.
+ * @param Client The client.
+ * @param User User name to set.
+ */
 GLOBAL void
-Client_SetInfo( CLIENT *Client, char *Info )
+Client_SetOrigUser(CLIENT UNUSED *Client, const char UNUSED *User)
 {
-	/* Hostname eines Clients setzen */
+	assert(Client != NULL);
+	assert(User != NULL);
+
+#if defined(PAM) && defined(IDENTAUTH)
+	strlcpy(Client->orig_user, User, sizeof(Client->orig_user));
+#endif
+} /* Client_SetOrigUser */
+
+
+GLOBAL void
+Client_SetInfo( CLIENT *Client, const char *Info )
+{
+	/* set client hostname */
 
 	assert( Client != NULL );
 	assert( Info != NULL );
 
-	strlcpy( Client->info, Info, sizeof( Client->info ));
+	if (Conf_CloakUserToNick)
+		strlcpy(Client->info, Client->id, sizeof(Client->info));
+	else
+		strlcpy(Client->info, Info, sizeof(Client->info));
 } /* Client_SetInfo */
 
 
 GLOBAL void
-Client_SetModes( CLIENT *Client, char *Modes )
+Client_SetModes( CLIENT *Client, const char *Modes )
 {
-	/* Modes eines Clients setzen */
-
 	assert( Client != NULL );
 	assert( Modes != NULL );
 
-	strlcpy( Client->modes, Modes, sizeof( Client->modes ));
+	strlcpy(Client->modes, Modes, sizeof( Client->modes ));
 } /* Client_SetModes */
 
 
 GLOBAL void
-Client_SetFlags( CLIENT *Client, char *Flags )
+Client_SetFlags( CLIENT *Client, const char *Flags )
 {
-	/* Flags eines Clients setzen */
-
 	assert( Client != NULL );
 	assert( Flags != NULL );
 
-	strlcpy( Client->flags, Flags, sizeof( Client->flags ));
+	strlcpy(Client->flags, Flags, sizeof(Client->flags));
 } /* Client_SetFlags */
 
 
 GLOBAL void
-Client_SetPassword( CLIENT *Client, char *Pwd )
+Client_SetPassword( CLIENT *Client, const char *Pwd )
 {
-	/* Von einem Client geliefertes Passwort */
+	/* set password sent by client */
 
 	assert( Client != NULL );
 	assert( Pwd != NULL );
 
-	strlcpy( Client->pwd, Pwd, sizeof( Client->pwd ));
+	strlcpy(Client->pwd, Pwd, sizeof(Client->pwd));
 } /* Client_SetPassword */
 
 
 GLOBAL void
-Client_SetAway( CLIENT *Client, char *Txt )
+Client_SetAway( CLIENT *Client, const char *Txt )
 {
 	/* Set AWAY reason of client */
 
@@ -432,7 +455,8 @@ Client_SetAway( CLIENT *Client, char *Txt )
 	assert( Txt != NULL );
 
 	strlcpy( Client->away, Txt, sizeof( Client->away ));
-	Log( LOG_DEBUG, "User \"%s\" is away: %s", Client_Mask( Client ), Txt );
+	LogDebug("%s \"%s\" is away: %s", Client_TypeText(Client),
+		 Client_Mask(Client), Txt);
 } /* Client_SetAway */
 
 
@@ -492,9 +516,7 @@ Client_ModeAdd( CLIENT *Client, char Mode )
 	assert( Client != NULL );
 
 	x[0] = Mode; x[1] = '\0';
-	if( ! strchr( Client->modes, x[0] ))
-	{
-		/* Client hat den Mode noch nicht -> setzen */
+	if (!strchr( Client->modes, x[0])) {
 		strlcat( Client->modes, x, sizeof( Client->modes ));
 		return true;
 	}
@@ -530,7 +552,7 @@ Client_ModeDel( CLIENT *Client, char Mode )
 
 
 GLOBAL CLIENT *
-Client_Search( char *Nick )
+Client_Search( const char *Nick )
 {
 	/* return Client-Structure that has the corresponding Nick.
 	 * If none is found, return NULL.
@@ -547,38 +569,37 @@ Client_Search( char *Nick )
 	ptr = strchr( search_id, '!' );
 	if( ptr ) *ptr = '\0';
 
-	search_hash = Hash( search_id );
+	search_hash = Hash(search_id);
 
 	c = My_Clients;
-	while( c )
-	{
-		if( c->hash == search_hash )
-		{
-			/* lt. Hash-Wert: Treffer! */
-			if( strcasecmp( c->id, search_id ) == 0 ) return c;
-		}
+	while (c) {
+		if (c->hash == search_hash && strcasecmp(c->id, search_id) == 0)
+			return c;
 		c = (CLIENT *)c->next;
 	}
 	return NULL;
 } /* Client_Search */
 
 
+/**
+ * Get client structure ("introducer") identfied by a server token.
+ * @return CLIENT structure or NULL if none could be found.
+ */
 GLOBAL CLIENT *
 Client_GetFromToken( CLIENT *Client, int Token )
 {
-	/* Client-Struktur, die den entsprechenden Introducer (=Client)
-	 * und das gegebene Token hat, liefern. Wird keine gefunden,
-	 * so wird NULL geliefert. */
-
 	CLIENT *c;
 
 	assert( Client != NULL );
-	assert( Token > 0 );
+
+	if (!Token)
+		return NULL;
 
 	c = My_Clients;
-	while( c )
-	{
-		if(( c->type == CLIENT_SERVER ) && ( c->introducer == Client ) && ( c->token == Token )) return c;
+	while (c) {
+		if ((c->type == CLIENT_SERVER) && (c->introducer == Client) &&
+			(c->token == Token))
+				return c;
 		c = (CLIENT *)c->next;
 	}
 	return NULL;
@@ -632,14 +653,61 @@ Client_User( CLIENT *Client )
 } /* Client_User */
 
 
+#ifdef PAM
+
+/**
+ * Get the "original" user name as supplied by the USER command.
+ * The user name as given by the client is used for authentication instead
+ * of the one detected using IDENT requests.
+ * @param Client The client.
+ * @return Original user name.
+ */
 GLOBAL char *
-Client_Hostname( CLIENT *Client )
+Client_OrigUser(CLIENT *Client) {
+#ifndef IDENTAUTH
+	char *user = Client->user;
+
+	if (user[0] == '~')
+		user++;
+	return user;
+#else
+	return Client->orig_user;
+#endif
+} /* Client_OrigUser */
+
+#endif
+
+
+/**
+ * Return the hostname of a client.
+ * @param Client Pointer to client structure
+ * @return Pointer to client hostname
+ */
+GLOBAL char *
+Client_Hostname(CLIENT *Client)
 {
-	assert( Client != NULL );
+	assert (Client != NULL);
 	return Client->host;
 } /* Client_Hostname */
 
 
+/**
+ * Get potentially cloaked hostname of a client.
+ * If the client has not enabled cloaking, the real hostname is used.
+ * @param Client Pointer to client structure
+ * @return Pointer to client hostname
+ */
+GLOBAL char *
+Client_HostnameCloaked(CLIENT *Client)
+{
+	assert(Client != NULL);
+	if (Client_HasMode(Client, 'x'))
+		return Client_ID(Client->introducer);
+	else
+		return Client_Hostname(Client);
+} /* Client_HostnameCloaked */
+
+
 GLOBAL char *
 Client_Password( CLIENT *Client )
 {
@@ -711,21 +779,59 @@ Client_NextHop( CLIENT *Client )
 } /* Client_NextHop */
 
 
+/**
+ * Return ID of a client: "client!user@host"
+ * This client ID is used for IRC prefixes, for example.
+ * Please note that this function uses a global static buffer, so you can't
+ * nest invocations without overwriting earlier results!
+ * @param Client Pointer to client structure
+ * @return Pointer to global buffer containing the client ID
+ */
 GLOBAL char *
 Client_Mask( CLIENT *Client )
 {
-	/* Client-"ID" liefern, wie sie z.B. fuer
-	 * Prefixe benoetigt wird. */
+	static char Mask_Buffer[GETID_LEN];
 
-	assert( Client != NULL );
+	assert (Client != NULL);
 
-	if( Client->type == CLIENT_SERVER ) return Client->id;
+	/* Servers: return name only, there is no "mask" */
+	if (Client->type == CLIENT_SERVER)
+		return Client->id;
 
-	snprintf( GetID_Buffer, GETID_LEN, "%s!%s@%s", Client->id, Client->user, Client->host );
-	return GetID_Buffer;
+	snprintf(Mask_Buffer, GETID_LEN, "%s!%s@%s",
+		 Client->id, Client->user, Client->host);
+	return Mask_Buffer;
 } /* Client_Mask */
 
 
+/**
+ * Return ID of a client with cloaked hostname: "client!user@server-name"
+ * This client ID is used for IRC prefixes, for example.
+ * Please note that this function uses a global static buffer, so you can't
+ * nest invocations without overwriting earlier results!
+ * If the client has not enabled cloaking, the real hostname is used.
+ * @param Client Pointer to client structure
+ * @return Pointer to global buffer containing the client ID
+ */
+GLOBAL char *
+Client_MaskCloaked(CLIENT *Client)
+{
+	static char Mask_Buffer[GETID_LEN];
+
+	assert (Client != NULL);
+
+	/* Is the client using cloaking at all? */
+	if (!Client_HasMode(Client, 'x'))
+	    return Client_Mask(Client);
+
+	snprintf(Mask_Buffer, GETID_LEN, "%s!%s@%s",
+		 Client->id, Client->user,
+		 *Conf_CloakHostModeX ? Conf_CloakHostModeX
+				      : Client_ID(Client->introducer));
+	return Mask_Buffer;
+} /* Client_MaskCloaked */
+
+
 GLOBAL CLIENT *
 Client_Introducer( CLIENT *Client )
 {
@@ -753,30 +859,42 @@ Client_HasMode( CLIENT *Client, char Mode )
 GLOBAL char *
 Client_Away( CLIENT *Client )
 {
-	/* AWAY-Text liefern */
-
 	assert( Client != NULL );
 	return Client->away;
 } /* Client_Away */
 
 
+/**
+ * Make sure that a given nickname is valid.
+ *
+ * If the nickname is not valid for the given client, this function sends back
+ * the appropriate error messages.
+ *
+ * @param	Client Client that wants to change the nickname.
+ * @param	Nick New nick name.
+ * @returns	true if nickname is valid, false otherwise.
+ */
 GLOBAL bool
-Client_CheckNick( CLIENT *Client, char *Nick )
+Client_CheckNick(CLIENT *Client, char *Nick)
 {
-	assert( Client != NULL );
-	assert( Nick != NULL );
+	assert(Client != NULL);
+	assert(Nick != NULL);
 
-	if( ! Client_IsValidNick( Nick ))
-	{
-		IRC_WriteStrClient( Client, ERR_ERRONEUSNICKNAME_MSG, Client_ID( Client ), Nick );
+	if (!Client_IsValidNick(Nick)) {
+		if (strlen(Nick ) >= Conf_MaxNickLength)
+			IRC_WriteStrClient(Client, ERR_NICKNAMETOOLONG_MSG,
+					   Client_ID(Client), Nick,
+					   Conf_MaxNickLength - 1);
+		else
+			IRC_WriteStrClient(Client, ERR_ERRONEUSNICKNAME_MSG,
+					   Client_ID(Client), Nick);
 		return false;
 	}
 
-	/* Nick bereits vergeben? */
-	if( Client_Search( Nick ))
-	{
-		/* den Nick gibt es bereits */
-		IRC_WriteStrClient( Client, ERR_NICKNAMEINUSE_MSG, Client_ID( Client ), Nick );
+	/* Nickname already registered? */
+	if (Client_Search(Nick)) {
+		IRC_WriteStrClient(Client, ERR_NICKNAMEINUSE_MSG,
+			Client_ID(Client), Nick);
 		return false;
 	}
 
@@ -787,8 +905,6 @@ Client_CheckNick( CLIENT *Client, char *Nick )
 GLOBAL bool
 Client_CheckID( CLIENT *Client, char *ID )
 {
-	/* Nick ueberpruefen */
-
 	char str[COMMAND_LEN];
 	CLIENT *c;
 
@@ -796,24 +912,22 @@ Client_CheckID( CLIENT *Client, char *ID )
 	assert( Client->conn_id > NONE );
 	assert( ID != NULL );
 
-	/* Nick zu lang? */
-	if( strlen( ID ) > CLIENT_ID_LEN )
-	{
-		IRC_WriteStrClient( Client, ERR_ERRONEUSNICKNAME_MSG, Client_ID( Client ), ID );
+	/* ID too long? */
+	if (strlen(ID) > CLIENT_ID_LEN) {
+		IRC_WriteStrClient(Client, ERR_ERRONEUSNICKNAME_MSG, Client_ID(Client), ID);
 		return false;
 	}
 
-	/* ID bereits vergeben? */
+	/* ID already in use? */
 	c = My_Clients;
-	while( c )
-	{
-		if( strcasecmp( c->id, ID ) == 0 )
-		{
-			/* die Server-ID gibt es bereits */
-			snprintf( str, sizeof( str ), "ID \"%s\" already registered", ID );
-			if( Client->conn_id != c->conn_id ) Log( LOG_ERR, "%s (on connection %d)!", str, c->conn_id );
-			else Log( LOG_ERR, "%s (via network)!", str );
-			Conn_Close( Client->conn_id, str, str, true);
+	while (c) {
+		if (strcasecmp(c->id, ID) == 0) {
+			snprintf(str, sizeof(str), "ID \"%s\" already registered", ID);
+			if (c->conn_id != NONE)
+				Log(LOG_ERR, "%s (on connection %d)!", str, c->conn_id);
+			else
+				Log(LOG_ERR, "%s (via network)!", str);
+			Conn_Close(Client->conn_id, str, str, true);
 			return false;
 		}
 		c = (CLIENT *)c->next;
@@ -826,8 +940,6 @@ Client_CheckID( CLIENT *Client, char *ID )
 GLOBAL CLIENT *
 Client_First( void )
 {
-	/* Ersten Client liefern. */
-
 	return My_Clients;
 } /* Client_First */
 
@@ -835,9 +947,6 @@ Client_First( void )
 GLOBAL CLIENT *
 Client_Next( CLIENT *c )
 {
-	/* Naechsten Client liefern. Existiert keiner,
-	 * so wird NULL geliefert. */
-
 	assert( c != NULL );
 	return (CLIENT *)c->next;
 } /* Client_Next */
@@ -941,23 +1050,31 @@ Client_MyMaxUserCount( void )
 } /* Client_MyMaxUserCount */
 
 
+/**
+ * Check that a given nickname is valid.
+ *
+ * @param	Nick the nickname to check.
+ * @returns	true if nickname is valid, false otherwise.
+ */
 GLOBAL bool
-Client_IsValidNick( const char *Nick )
+Client_IsValidNick(const char *Nick)
 {
 	const char *ptr;
 	static const char goodchars[] = ";0123456789-";
 
-	assert( Nick != NULL );
+	assert (Nick != NULL);
 
-	if( Nick[0] == '#' ) return false;
-	if( strchr( goodchars, Nick[0] )) return false;
-	if( strlen( Nick ) >= Conf_MaxNickLength) return false;
+	if (strchr(goodchars, Nick[0]))
+		return false;
+	if (strlen(Nick ) >= Conf_MaxNickLength)
+		return false;
 
 	ptr = Nick;
-	while( *ptr )
-	{
-		if (( *ptr < 'A' ) && ( ! strchr( goodchars, *ptr ))) return false;
-		if ( *ptr > '}' ) return false;
+	while (*ptr) {
+		if (*ptr < 'A' && !strchr(goodchars, *ptr ))
+			return false;
+		if (*ptr > '}')
+			return false;
 		ptr++;
 	}
 
@@ -997,6 +1114,79 @@ Client_StartTime(CLIENT *Client)
 } /* Client_Uptime */
 
 
+/**
+ * Reject a client when logging in.
+ *
+ * This function is called when a client isn't allowed to connect to this
+ * server. Possible reasons are bad server password, bad PAM password,
+ * or that the client is G/K-Line'd.
+ *
+ * After calling this function, the client isn't connected any more.
+ *
+ * @param Client The client to reject.
+ * @param Reason The reason why the client has been rejected.
+ * @param InformClient If true, send the exact reason to the client.
+ */
+GLOBAL void
+Client_Reject(CLIENT *Client, const char *Reason, bool InformClient)
+{
+	char info[COMMAND_LEN];
+
+	assert(Client != NULL);
+	assert(Reason != NULL);
+
+	if (InformClient)
+		snprintf(info, sizeof(info), "Access denied: %s", Reason);
+	else
+		strcpy(info, "Access denied: Bad password?");
+
+	Log(LOG_ERR,
+	    "User \"%s\" rejected (connection %d): %s!",
+	    Client_Mask(Client), Client_Conn(Client), Reason);
+	Conn_Close(Client_Conn(Client), Reason, info, true);
+}
+
+
+/**
+ * Introduce a new user or service client in the network.
+ *
+ * @param From Remote server introducing the client or NULL (local).
+ * @param Client New client.
+ * @param Type Type of the client (CLIENT_USER or CLIENT_SERVICE).
+ */
+GLOBAL void
+Client_Introduce(CLIENT *From, CLIENT *Client, int Type)
+{
+	/* Set client type (user or service) */
+	Client_SetType(Client, Type);
+
+	if (From) {
+		if (Conf_IsService(Conf_GetServer(Client_Conn(From)),
+				   Client_ID(Client)))
+			Client_SetType(Client, CLIENT_SERVICE);
+		LogDebug("%s \"%s\" (+%s) registered (via %s, on %s, %d hop%s).",
+			 Client_TypeText(Client), Client_Mask(Client),
+			 Client_Modes(Client), Client_ID(From),
+			 Client_ID(Client_Introducer(Client)),
+			 Client_Hops(Client), Client_Hops(Client) > 1 ? "s": "");
+	} else {
+		Log(LOG_NOTICE, "%s \"%s\" registered (connection %d).",
+		    Client_TypeText(Client), Client_Mask(Client),
+		    Client_Conn(Client));
+		Log_ServerNotice('c', "Client connecting: %s (%s@%s) [%s] - %s",
+			         Client_ID(Client), Client_User(Client),
+				 Client_Hostname(Client),
+				 Conn_IPA(Client_Conn(Client)),
+				 Client_TypeText(Client));
+	}
+
+	/* Inform other servers */
+	IRC_WriteStrServersPrefixFlag_CB(From,
+				From != NULL ? From : Client_ThisServer(),
+				'\0', cb_introduceClient, (void *)Client);
+} /* Client_Introduce */
+
+
 static unsigned long
 Count( CLIENT_TYPE Type )
 {
@@ -1076,7 +1266,7 @@ Generate_MyToken( CLIENT *Client )
 		else c = (CLIENT *)c->next;
 	}
 	Client->mytoken = token;
-	Log( LOG_DEBUG, "Assigned token %d to server \"%s\".", token, Client->id );
+	LogDebug("Assigned token %d to server \"%s\".", token, Client->id);
 } /* Generate_MyToken */
 
 
@@ -1114,6 +1304,10 @@ Client_RegisterWhowas( CLIENT *Client )
 
 	assert( Client != NULL );
 
+	/* Don't register WHOWAS information when "MorePrivacy" is enabled. */
+	if (Conf_MorePrivacy)
+		return;
+
 	now = time(NULL);
 	/* Don't register clients that were connected less than 30 seconds. */
 	if( now - Client->starttime < 30 )
@@ -1131,7 +1325,7 @@ Client_RegisterWhowas( CLIENT *Client )
 		 sizeof( My_Whowas[slot].id ));
 	strlcpy( My_Whowas[slot].user, Client_User( Client ),
 		 sizeof( My_Whowas[slot].user ));
-	strlcpy( My_Whowas[slot].host, Client_Hostname( Client ),
+	strlcpy( My_Whowas[slot].host, Client_HostnameCloaked( Client ),
 		 sizeof( My_Whowas[slot].host ));
 	strlcpy( My_Whowas[slot].info, Client_Info( Client ),
 		 sizeof( My_Whowas[slot].info ));
@@ -1142,4 +1336,149 @@ Client_RegisterWhowas( CLIENT *Client )
 } /* Client_RegisterWhowas */
 
 
+GLOBAL const char *
+Client_TypeText(CLIENT *Client)
+{
+	assert(Client != NULL);
+	switch (Client_Type(Client)) {
+		case CLIENT_USER:
+			return "User";
+			break;
+		case CLIENT_SERVICE:
+			return "Service";
+			break;
+		case CLIENT_SERVER:
+			return "Server";
+			break;
+		default:
+			return "Client";
+	}
+} /* Client_TypeText */
+
+
+/**
+ * Destroy user or service client.
+ */
+static void
+Destroy_UserOrService(CLIENT *Client, const char *Txt, const char *FwdMsg, bool SendQuit)
+{
+	if(Client->conn_id != NONE) {
+		/* Local (directly connected) client */
+		Log(LOG_NOTICE,
+		    "%s \"%s\" unregistered (connection %d): %s",
+		    Client_TypeText(Client), Client_Mask(Client),
+		    Client->conn_id, Txt);
+		Log_ServerNotice('c', "Client exiting: %s (%s@%s) [%s]",
+			         Client_ID(Client), Client_User(Client),
+				 Client_Hostname(Client), Txt);
+
+		if (SendQuit) {
+			/* Inforam all the other servers */
+			if (FwdMsg)
+				IRC_WriteStrServersPrefix(NULL,
+						Client, "QUIT :%s", FwdMsg );
+			else
+				IRC_WriteStrServersPrefix(NULL,
+						Client, "QUIT :");
+		}
+	} else {
+		/* Remote client */
+		LogDebug("%s \"%s\" unregistered: %s",
+			 Client_TypeText(Client), Client_Mask(Client), Txt);
+
+		if(SendQuit) {
+			/* Inform all the other servers, but the ones in the
+			 * direction we got the QUIT from */
+			if(FwdMsg)
+				IRC_WriteStrServersPrefix(Client_NextHop(Client),
+						Client, "QUIT :%s", FwdMsg );
+			else
+				IRC_WriteStrServersPrefix(Client_NextHop(Client),
+						Client, "QUIT :" );
+		}
+	}
+
+	/* Unregister client from channels */
+	Channel_Quit(Client, FwdMsg ? FwdMsg : Client->id);
+
+	/* Register client in My_Whowas structure */
+	Client_RegisterWhowas(Client);
+} /* Destroy_UserOrService */
+
+
+/**
+ * Introduce a new user or service client to a remote server.
+ *
+ * This function differentiates between RFC1459 and RFC2813 server links and
+ * generates the appropriate commands to register the new user or service.
+ *
+ * @param To		The remote server to inform.
+ * @param Prefix	Prefix for the generated commands.
+ * @param data		CLIENT structure of the new client.
+ */
+static void
+cb_introduceClient(CLIENT *To, CLIENT *Prefix, void *data)
+{
+	CLIENT *c = (CLIENT *)data;
+	CONN_ID conn;
+	char *modes, *user, *host;
+
+	modes = Client_Modes(c);
+	user = Client_User(c) ? Client_User(c) : "-";
+	host = Client_Hostname(c) ? Client_Hostname(c) : "-";
+
+	conn = Client_Conn(To);
+	if (Conn_Options(conn) & CONN_RFC1459) {
+		/* RFC 1459 mode: separate NICK and USER commands */
+		Conn_WriteStr(conn, "NICK %s :%d", Client_ID(c),
+			      Client_Hops(c) + 1);
+		Conn_WriteStr(conn, ":%s USER %s %s %s :%s",
+			      Client_ID(c), user, host,
+			      Client_ID(Client_Introducer(c)), Client_Info(c));
+		if (modes[0])
+			Conn_WriteStr(conn, ":%s MODE %s +%s",
+				      Client_ID(c), Client_ID(c), modes);
+	} else {
+		/* RFC 2813 mode: one combined NICK or SERVICE command */
+		if (Client_Type(c) == CLIENT_SERVICE
+		    && strchr(Client_Flags(To), 'S'))
+			IRC_WriteStrClientPrefix(To, Prefix,
+						 "SERVICE %s %d * +%s %d :%s",
+						 Client_Mask(c),
+						 Client_MyToken(Client_Introducer(c)),
+						 Client_Modes(c), Client_Hops(c) + 1,
+						 Client_Info(c));
+		else
+			IRC_WriteStrClientPrefix(To, Prefix,
+						 "NICK %s %d %s %s %d +%s :%s",
+						 Client_ID(c), Client_Hops(c) + 1,
+						 user, host,
+						 Client_MyToken(Client_Introducer(c)),
+						 modes, Client_Info(c));
+	}
+} /* cb_introduceClient */
+
+
+#ifdef DEBUG
+
+GLOBAL void
+Client_DebugDump(void)
+{
+	CLIENT *c;
+
+	Log(LOG_DEBUG, "Client status:");
+	c = My_Clients;
+	while (c) {
+		Log(LOG_DEBUG,
+		    " - %s: type=%d, host=%s, user=%s, conn=%d, start=%ld, flags=%s",
+                   Client_ID(c), Client_Type(c), Client_Hostname(c),
+                   Client_User(c), Client_Conn(c), Client_StartTime(c),
+                   Client_Flags(c));
+		c = (CLIENT *)c->next;
+	}
+} /* Client_DumpClients */
+
+#endif
+
+
 /* -eof- */
