@@ -7,23 +7,22 @@
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  * Please read the file COPYING, README and AUTHORS for more information.
- *
- * Connection compression using ZLIB
  */
 
-
 #include "portab.h"
 
-#define CONN_MODULE
+/**
+ * @file
+ * Connection compression using ZLIB
+ */
 
+#define CONN_MODULE
 
 #ifdef ZLIB
 
 /* enable more zlib related debug messages: */
 /* #define DEBUG_ZLIB */
 
-static char UNUSED id[] = "$Id: conn-zip.c,v 1.16 2007/05/17 23:34:24 alex Exp $";
-
 #include "imp.h"
 #include <assert.h>
 #include <string.h>
@@ -41,7 +40,7 @@ static char UNUSED id[] = "$Id: conn-zip.c,v 1.16 2007/05/17 23:34:24 alex Exp $
 GLOBAL bool
 Zip_InitConn( CONN_ID Idx )
 {
-	/* Kompression fuer Link initialisieren */
+	/* initialize zlib compression on this link */
 
 	assert( Idx > NONE );
 
@@ -52,10 +51,8 @@ Zip_InitConn( CONN_ID Idx )
 	My_Connections[Idx].zip.in.zfree = NULL;
 	My_Connections[Idx].zip.in.data_type = Z_ASCII;
 
-	if( inflateInit( &My_Connections[Idx].zip.in ) != Z_OK )
-	{
-		/* Fehler! */
-		Log( LOG_ALERT, "Can't initialize compression on connection %d (zlib inflate)!", Idx );
+	if (inflateInit( &My_Connections[Idx].zip.in ) != Z_OK) {
+		Log(LOG_ALERT, "Can't initialize compression on connection %d (zlib inflate)!", Idx);
 		return false;
 	}
 
@@ -65,17 +62,15 @@ Zip_InitConn( CONN_ID Idx )
 	My_Connections[Idx].zip.out.zfree = NULL;
 	My_Connections[Idx].zip.out.data_type = Z_ASCII;
 
-	if( deflateInit( &My_Connections[Idx].zip.out, Z_DEFAULT_COMPRESSION ) != Z_OK )
-	{
-		/* Fehler! */
-		Log( LOG_ALERT, "Can't initialize compression on connection %d (zlib deflate)!", Idx );
+	if (deflateInit( &My_Connections[Idx].zip.out, Z_DEFAULT_COMPRESSION ) != Z_OK) {
+		Log(LOG_ALERT, "Can't initialize compression on connection %d (zlib deflate)!", Idx);
 		return false;
 	}
 
 	My_Connections[Idx].zip.bytes_in = My_Connections[Idx].bytes_in;
 	My_Connections[Idx].zip.bytes_out = My_Connections[Idx].bytes_out;
 
-	Log( LOG_INFO, "Enabled link compression (zlib) on connection %d.", Idx );
+	Log(LOG_INFO, "Enabled link compression (zlib) on connection %d.", Idx);
 	Conn_OPTION_ADD( &My_Connections[Idx], CONN_ZIP );
 
 	return true;
@@ -88,12 +83,14 @@ Zip_InitConn( CONN_ID Idx )
  * compression ratios.
  * If the (pre-)compression buffer is full, we try to flush it ("actually
  * compress some data") and to add the new (uncompressed) data afterwards.
+ * This function closes the connection on error.
  * @param Idx Connection handle.
  * @param Data Pointer to the data.
  * @param Len Length of the data to add.
- * @return true on success, false otherwise. */
+ * @return true on success, false otherwise.
+ */
 GLOBAL bool
-Zip_Buffer( CONN_ID Idx, char *Data, size_t Len )
+Zip_Buffer( CONN_ID Idx, const char *Data, size_t Len )
 {
 	size_t buflen;
 
@@ -110,9 +107,11 @@ Zip_Buffer( CONN_ID Idx, char *Data, size_t Len )
 	/* check again; if zip buf is still too large do not append data:
 	 * otherwise the zip wbuf would grow too large */
 	buflen = array_bytes(&My_Connections[Idx].zip.wbuf);
-	if (buflen + Len >= WRITEBUFFER_SLINK_LEN)
+	if (buflen + Len >= WRITEBUFFER_SLINK_LEN) {
+		Log(LOG_ALERT, "Zip Write buffer space exhausted: %lu bytes", buflen + Len);
+		Conn_Close(Idx, "Zip Write buffer space exhausted", NULL, false);
 		return false;
-
+	}
 	return array_catb(&My_Connections[Idx].zip.wbuf, Data, Len);
 } /* Zip_Buffer */
 
@@ -120,8 +119,9 @@ Zip_Buffer( CONN_ID Idx, char *Data, size_t Len )
 /**
  * Compress data in ZIP buffer and move result to the write buffer of
  * the connection.
+ * This function closes the connection on error.
  * @param Idx Connection handle.
- * @retrun true on success, false otherwise.
+ * @return true on success, false otherwise.
  */
 GLOBAL bool
 Zip_Flush( CONN_ID Idx )
@@ -158,7 +158,7 @@ Zip_Flush( CONN_ID Idx )
 	if (out->avail_out <= 0) {
 		/* Not all data was compressed, because data became
 		 * bigger while compressing it. */
-		Log (LOG_ALERT, "Compression error: buffer overvlow!?");
+		Log(LOG_ALERT, "Compression error: buffer overflow!?");
 		Conn_Close(Idx, "Compression error!", NULL, false);
 		return false;
 	}
@@ -184,13 +184,17 @@ Zip_Flush( CONN_ID Idx )
 } /* Zip_Flush */
 
 
+/**
+ * uncompress data and copy it to read buffer.
+ * Returns true if data has been unpacked or no
+ * compressed data is currently pending in the zread buffer.
+ * This function closes the connection on error.
+ * @param Idx Connection handle.
+ * @return true on success, false otherwise.
+ */
 GLOBAL bool
 Unzip_Buffer( CONN_ID Idx )
 {
-	/* Daten entpacken und in Lesepuffer kopieren. Bei Fehlern
-	* wird false geliefert, ansonsten true. Der Fall, dass keine
-	* Daten mehr zu entpacken sind, ist _kein_ Fehler! */
-
 	int result;
 	unsigned char unzipbuf[READBUFFER_LEN];
 	int unzipbuf_used = 0;
@@ -221,8 +225,8 @@ Unzip_Buffer( CONN_ID Idx )
 	result = inflate( in, Z_SYNC_FLUSH );
 	if( result != Z_OK )
 	{
-		Log( LOG_ALERT, "Decompression error: %s (code=%d, ni=%d, ai=%d, no=%d, ao=%d)!?", in->msg, result, in->next_in, in->avail_in, in->next_out, in->avail_out );
-		Conn_Close( Idx, "Decompression error!", NULL, false );
+		Log(LOG_ALERT, "Decompression error: %s (code=%d, ni=%d, ai=%d, no=%d, ao=%d)!?", in->msg, result, in->next_in, in->avail_in, in->next_out, in->avail_out);
+		Conn_Close(Idx, "Decompression error!", NULL, false);
 		return false;
 	}
 
@@ -235,9 +239,11 @@ Unzip_Buffer( CONN_ID Idx )
 #endif
 	assert(unzipbuf_used <= READBUFFER_LEN);
 	if (!array_catb(&My_Connections[Idx].rbuf, (char*) unzipbuf,
-			(size_t)unzipbuf_used))
+			(size_t)unzipbuf_used)) {
+		Log (LOG_ALERT, "Decompression error: can't copy data!?");
+		Conn_Close(Idx, "Decompression error!", NULL, false);
 		return false;
-
+	}
 	if( in->avail_in > 0 ) {
 		array_moveleft(&My_Connections[Idx].zip.rbuf, 1, in_len );
 	} else {
@@ -249,21 +255,25 @@ Unzip_Buffer( CONN_ID Idx )
 } /* Unzip_Buffer */
 
 
+/**
+ * @param Idx Connection handle.
+ * @return amount of sent (compressed) bytes
+ */
 GLOBAL long
 Zip_SendBytes( CONN_ID Idx )
 {
-	/* Anzahl gesendeter Bytes (komprimiert!) liefern */
-
 	assert( Idx > NONE );
 	return My_Connections[Idx].zip.bytes_out;
 } /* Zip_SendBytes */
 
 
+/**
+ * @param Idx Connection handle.
+ * @return amount of received (compressed) bytes
+ */
 GLOBAL long
 Zip_RecvBytes( CONN_ID Idx )
 {
-	/* Anzahl gesendeter Bytes (komprimiert!) liefern */
-
 	assert( Idx > NONE );
 	return My_Connections[Idx].zip.bytes_in;
 } /* Zip_RecvBytes */
