@@ -1,20 +1,20 @@
 /*
  * ngIRCd -- The Next Generation IRC Daemon
- * Copyright (c)2001-2005 by Alexander Barton (alex@barton.de)
+ * Copyright (c)2001-2008 Alexander Barton (alex@barton.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  * Please read the file COPYING, README and AUTHORS for more information.
- *
- * Sending IRC commands over the network
  */
 
-
 #include "portab.h"
 
-static char UNUSED id[] = "$Id: irc-write.c,v 1.21 2006/08/12 11:56:24 fw Exp $";
+/**
+ * @file
+ * Sending IRC commands over the network
+ */
 
 #include "imp.h"
 #include <assert.h>
@@ -28,7 +28,6 @@ static char UNUSED id[] = "$Id: irc-write.c,v 1.21 2006/08/12 11:56:24 fw Exp $"
 
 #include "defines.h"
 #include "conn-func.h"
-#include "client.h"
 #include "channel.h"
 
 #include "exp.h"
@@ -39,17 +38,20 @@ static char UNUSED id[] = "$Id: irc-write.c,v 1.21 2006/08/12 11:56:24 fw Exp $"
 #define SEND_TO_SERVER 2
 
 
-static char *Get_Prefix PARAMS(( CLIENT *Target, CLIENT *Client ));
+static const char *Get_Prefix PARAMS((CLIENT *Target, CLIENT *Client));
+static void cb_writeStrServersPrefixFlag PARAMS((CLIENT *Client,
+					 CLIENT *Prefix, void *Buffer));
+static bool Send_Marked_Connections PARAMS((CLIENT *Prefix, const char *Buffer));
 
 
 #ifdef PROTOTYPES
 GLOBAL bool
-IRC_WriteStrClient( CLIENT *Client, char *Format, ... )
+IRC_WriteStrClient( CLIENT *Client, const char *Format, ... )
 #else
 GLOBAL bool
 IRC_WriteStrClient( Client, Format, va_alist )
 CLIENT *Client;
-char *Format;
+const char *Format;
 va_dcl
 #endif
 {
@@ -68,7 +70,7 @@ va_dcl
 	vsnprintf( buffer, 1000, Format, ap );
 	va_end( ap );
 
-	/* an den Client selber */
+	/* to the client itself */
 	ok = IRC_WriteStrClientPrefix( Client, Client_ThisServer( ), "%s", buffer );
 
 	return ok;
@@ -77,17 +79,17 @@ va_dcl
 
 #ifdef PROTOTYPES
 GLOBAL bool
-IRC_WriteStrClientPrefix( CLIENT *Client, CLIENT *Prefix, char *Format, ... )
+IRC_WriteStrClientPrefix(CLIENT *Client, CLIENT *Prefix, const char *Format, ...)
 #else
 GLOBAL bool
-IRC_WriteStrClientPrefix( Client, Prefix, Format, va_alist )
+IRC_WriteStrClientPrefix(Client, Prefix, Format, va_alist)
 CLIENT *Client;
 CLIENT *Prefix;
-char *Format;
+const char *Format;
 va_dcl
 #endif
 {
-	/* Text an Clients, lokal bzw. remote, senden. */
+	/* send text to local and remote clients */
 
 	char buffer[1000];
 	va_list ap;
@@ -104,20 +106,22 @@ va_dcl
 	vsnprintf( buffer, 1000, Format, ap );
 	va_end( ap );
 
-	return Conn_WriteStr( Client_Conn( Client_NextHop( Client )), ":%s %s", Get_Prefix( Client_NextHop( Client ), Prefix ), buffer );
+	return Conn_WriteStr(Client_Conn(Client_NextHop(Client)), ":%s %s",
+			Get_Prefix(Client_NextHop(Client), Prefix), buffer);
 } /* IRC_WriteStrClientPrefix */
 
 
 #ifdef PROTOTYPES
 GLOBAL bool
-IRC_WriteStrChannel( CLIENT *Client, CHANNEL *Chan, bool Remote, char *Format, ... )
+IRC_WriteStrChannel(CLIENT *Client, CHANNEL *Chan, bool Remote,
+		    const char *Format, ...)
 #else
 GLOBAL bool
-IRC_WriteStrChannel( Client, Chan, Remote, Format, va_alist )
+IRC_WriteStrChannel(Client, Chan, Remote, Format, va_alist)
 CLIENT *Client;
 CHANNEL *Chan;
 bool Remote;
-char *Format;
+const char *Format;
 va_dcl
 #endif
 {
@@ -139,21 +143,26 @@ va_dcl
 } /* IRC_WriteStrChannel */
 
 
+
+/**
+ * send message to all clients in the same channel, but only send message
+ * once per remote server.
+ */
 #ifdef PROTOTYPES
 GLOBAL bool
-IRC_WriteStrChannelPrefix( CLIENT *Client, CHANNEL *Chan, CLIENT *Prefix, bool Remote, char *Format, ... )
+IRC_WriteStrChannelPrefix(CLIENT *Client, CHANNEL *Chan, CLIENT *Prefix,
+			  bool Remote, const char *Format, ...)
 #else
 GLOBAL bool
-IRC_WriteStrChannelPrefix( Client, Chan, Prefix, Remote, Format, va_alist )
+IRC_WriteStrChannelPrefix(Client, Chan, Prefix, Remote, Format, va_alist)
 CLIENT *Client;
 CHANNEL *Chan;
 CLIENT *Prefix;
 bool Remote;
-char *Format;
+const char *Format;
 va_dcl
 #endif
 {
-	bool ok = CONNECTED;
 	char buffer[1000];
 	CL2CHAN *cl2chan;
 	CONN_ID conn;
@@ -175,8 +184,6 @@ va_dcl
 
 	Conn_ClearFlags( );
 
-	/* An alle Clients, die in den selben Channels sind.
-	 * Dabei aber nur einmal je Remote-Server */
 	cl2chan = Channel_FirstMember( Chan );
 	while( cl2chan )
 	{
@@ -187,42 +194,28 @@ va_dcl
 			else if( Client_Type( c ) == CLIENT_SERVER ) c = NULL;
 		}
 		if( c ) c = Client_NextHop( c );
-			
+
 		if( c && ( c != Client ))
 		{
-			/* Ok, anderer Client */
+			/* Ok, another Client */
 			conn = Client_Conn( c );
 			if( Client_Type( c ) == CLIENT_SERVER )	Conn_SetFlag( conn, SEND_TO_SERVER );
 			else Conn_SetFlag( conn, SEND_TO_USER );
 		}
 		cl2chan = Channel_NextMember( Chan, cl2chan );
 	}
-
-	/* Senden: alle Verbindungen durchgehen ... */
-	conn = Conn_First( );
-	while( conn != NONE )
-	{
-		/* muessen Daten ueber diese Verbindung verschickt werden? */
-		if( Conn_Flag( conn ) == SEND_TO_SERVER) ok = Conn_WriteStr( conn, ":%s %s", Client_ID( Prefix ), buffer );
-		else if( Conn_Flag( conn ) == SEND_TO_USER ) ok = Conn_WriteStr( conn, ":%s %s", Client_Mask( Prefix ), buffer );
-		if( ! ok ) break;
-
-		/* naechste Verbindung testen */
-		conn = Conn_Next( conn );
-	}
-
-	return ok;
+	return Send_Marked_Connections(Prefix, buffer);
 } /* IRC_WriteStrChannelPrefix */
 
 
 #ifdef PROTOTYPES
 GLOBAL void
-IRC_WriteStrServers( CLIENT *ExceptOf, char *Format, ... )
+IRC_WriteStrServers(CLIENT *ExceptOf, const char *Format, ...)
 #else
 GLOBAL void
-IRC_WriteStrServers( ExceptOf, Format, va_alist )
+IRC_WriteStrServers(ExceptOf, Format, va_alist)
 CLIENT *ExceptOf;
-char *Format;
+const char *Format;
 va_dcl
 #endif
 {
@@ -239,20 +232,20 @@ va_dcl
 	vsnprintf( buffer, 1000, Format, ap );
 	va_end( ap );
 
-	/* an den Client selber */
 	IRC_WriteStrServersPrefix( ExceptOf, Client_ThisServer( ), "%s", buffer );
 } /* IRC_WriteStrServers */
 
 
 #ifdef PROTOTYPES
 GLOBAL void
-IRC_WriteStrServersPrefix( CLIENT *ExceptOf, CLIENT *Prefix, char *Format, ... )
+IRC_WriteStrServersPrefix(CLIENT *ExceptOf, CLIENT *Prefix,
+			  const char *Format, ...)
 #else
 GLOBAL void
-IRC_WriteStrServersPrefix( ExceptOf, Prefix, Format, va_alist )
+IRC_WriteStrServersPrefix(ExceptOf, Prefix, Format, va_alist)
 CLIENT *ExceptOf;
 CLIENT *Prefix;
-char *Format;
+const char *Format;
 va_dcl
 #endif
 {
@@ -272,25 +265,25 @@ va_dcl
 
 	IRC_WriteStrServersPrefixFlag( ExceptOf, Prefix, '\0', "%s", buffer );
 } /* IRC_WriteStrServersPrefix */
-	
+
 
 #ifdef PROTOTYPES
 GLOBAL void
-IRC_WriteStrServersPrefixFlag( CLIENT *ExceptOf, CLIENT *Prefix, char Flag, char *Format, ... )
+IRC_WriteStrServersPrefixFlag(CLIENT *ExceptOf, CLIENT *Prefix, char Flag,
+			      const char *Format, ...)
 #else
 GLOBAL void
-IRC_WriteStrServersPrefixFlag( ExceptOf, Prefix, Flag, Format, va_alist )
+IRC_WriteStrServersPrefixFlag(ExceptOf, Prefix, Flag, Format, va_alist)
 CLIENT *ExceptOf;
 CLIENT *Prefix;
 char Flag;
-char *Format;
+const char *Format;
 va_dcl
 #endif
 {
 	char buffer[1000];
-	CLIENT *c;
 	va_list ap;
-	
+
 	assert( Format != NULL );
 	assert( Prefix != NULL );
 
@@ -301,34 +294,49 @@ va_dcl
 #endif
 	vsnprintf( buffer, 1000, Format, ap );
 	va_end( ap );
-	
-	c = Client_First( );
-	while( c )
-	{
-		if(( Client_Type( c ) == CLIENT_SERVER ) && ( Client_Conn( c ) > NONE ) && ( c != Client_ThisServer( )) && ( c != ExceptOf ))
-		{
-			/* Ziel-Server gefunden. Nun noch pruefen, ob Flags stimmen */
-			if(( Flag == '\0' ) || ( strchr( Client_Flags( c ), Flag ) != NULL )) IRC_WriteStrClientPrefix( c, Prefix, "%s", buffer );
+
+	IRC_WriteStrServersPrefixFlag_CB(ExceptOf, Prefix, Flag,
+					 cb_writeStrServersPrefixFlag, buffer);
+} /* IRC_WriteStrServersPrefixFlag */
+
+
+GLOBAL void
+IRC_WriteStrServersPrefixFlag_CB(CLIENT *ExceptOf, CLIENT *Prefix, char Flag,
+		void (*callback)(CLIENT *, CLIENT *, void *), void *cb_data)
+{
+	CLIENT *c;
+
+	c = Client_First();
+	while(c) {
+		if (Client_Type(c) == CLIENT_SERVER && Client_Conn(c) > NONE &&
+		    c != Client_ThisServer() && c != ExceptOf) {
+			/* Found a target server, do the flags match? */
+			if (Flag == '\0' || strchr(Client_Flags(c), Flag))
+				callback(c, Prefix, cb_data);
 		}
-		c = Client_Next( c );
+		c = Client_Next(c);
 	}
 } /* IRC_WriteStrServersPrefixFlag */
 
 
+/**
+ * send message to all clients that are in the same channels as the client sending this message.
+ * only send message once per reote server.
+ */
 #ifdef PROTOTYPES
 GLOBAL bool
-IRC_WriteStrRelatedPrefix( CLIENT *Client, CLIENT *Prefix, bool Remote, char *Format, ... )
+IRC_WriteStrRelatedPrefix(CLIENT *Client, CLIENT *Prefix, bool Remote,
+			  const char *Format, ...)
 #else
 GLOBAL bool
-IRC_WriteStrRelatedPrefix( Client, Prefix, Remote, Format, va_alist )
+IRC_WriteStrRelatedPrefix(Client, Prefix, Remote, Format, va_alist)
 CLIENT *Client;
 CLIENT *Prefix;
 bool Remote;
-char *Format;
+const char *Format;
 va_dcl
 #endif
 {
-	bool ok = CONNECTED;
 	CL2CHAN *chan_cl2chan, *cl2chan;
 	char buffer[1000];
 	CHANNEL *chan;
@@ -348,15 +356,11 @@ va_dcl
 	vsnprintf( buffer, 1000, Format, ap );
 	va_end( ap );
 
-	/* initialisieren */
 	Conn_ClearFlags( );
 
-	/* An alle Clients, die in einem Channel mit dem "Ausloeser" sind,
-	 * den Text schicken. An Remote-Server aber jeweils nur einmal. */
 	chan_cl2chan = Channel_FirstChannelOf( Client );
 	while( chan_cl2chan )
 	{
-		/* Channel des Users durchsuchen */
 		chan = Channel_GetChannel( chan_cl2chan );
 		cl2chan = Channel_FirstMember( chan );
 		while( cl2chan )
@@ -371,7 +375,6 @@ va_dcl
 
 			if( c && ( c != Client ))
 			{
-				/* Ok, anderer Client */
 				conn = Client_Conn( c );
 				if( Client_Type( c ) == CLIENT_SERVER ) Conn_SetFlag( conn, SEND_TO_SERVER );
 				else Conn_SetFlag( conn, SEND_TO_USER );
@@ -379,26 +382,59 @@ va_dcl
 			cl2chan = Channel_NextMember( chan, cl2chan );
 		}
 
-		/* naechsten Channel */
 		chan_cl2chan = Channel_NextChannelOf( Client, chan_cl2chan );
 	}
-
-	/* Senden: alle Verbindungen durchgehen ... */
-	conn = Conn_First( );
-	while( conn != NONE )
-	{
-		/* muessen ueber diese Verbindung Daten gesendet werden? */
-		if( Conn_Flag( conn ) == SEND_TO_SERVER ) ok = Conn_WriteStr( conn, ":%s %s", Client_ID( Prefix ), buffer );
-		else if( Conn_Flag( conn ) == SEND_TO_USER ) ok = Conn_WriteStr( conn, ":%s %s", Client_Mask( Prefix ), buffer );
-		if( ! ok ) break;
-
-		/* naechste Verbindung testen */
-		conn = Conn_Next( conn );
-	}
-	return ok;
+	return Send_Marked_Connections(Prefix, buffer);
 } /* IRC_WriteStrRelatedPrefix */
 
 
+/**
+ * Send WALLOPS message.
+ */
+#ifdef PROTOTYPES
+GLOBAL void
+IRC_SendWallops(CLIENT *Client, CLIENT *From, const char *Format, ...)
+#else
+GLOBAL void
+IRC_SendWallops(Client, From, Format, va_alist )
+CLIENT *Client;
+CLIENT *From;
+const char *Format;
+va_dcl
+#endif
+{
+	va_list ap;
+	char msg[1000];
+	CLIENT *to;
+
+#ifdef PROTOTYPES
+	va_start(ap, Format);
+#else
+	va_start(ap);
+#endif
+	vsnprintf(msg, 1000, Format, ap);
+	va_end(ap);
+
+	for (to=Client_First(); to != NULL; to=Client_Next(to)) {
+		if (Client_Conn(to) == NONE) /* no local connection */
+			continue;
+
+		switch (Client_Type(to)) {
+		case CLIENT_USER:
+			if (Client_HasMode(to, 'w'))
+				IRC_WriteStrClientPrefix(to, From,
+							 "WALLOPS :%s", msg);
+				break;
+		case CLIENT_SERVER:
+			if (to != Client)
+				IRC_WriteStrClientPrefix(to, From,
+							 "WALLOPS :%s", msg);
+				break;
+		}
+	}
+} /* IRC_SendWallops */
+
+
 GLOBAL void
 IRC_SetPenalty( CLIENT *Client, time_t Seconds )
 {
@@ -415,15 +451,49 @@ IRC_SetPenalty( CLIENT *Client, time_t Seconds )
 } /* IRC_SetPenalty */
 
 
-static char *
-Get_Prefix( CLIENT *Target, CLIENT *Client )
+static const char *
+Get_Prefix(CLIENT *Target, CLIENT *Client)
 {
-	assert( Target != NULL );
-	assert( Client != NULL );
+	assert (Target != NULL);
+	assert (Client != NULL);
 
-	if( Client_Type( Target ) == CLIENT_SERVER ) return Client_ID( Client );
-	else return Client_Mask( Client );
+	if (Client_Type(Target) == CLIENT_SERVER)
+		return Client_ID(Client);
+	else
+		return Client_MaskCloaked(Client);
 } /* Get_Prefix */
 
 
+static void
+cb_writeStrServersPrefixFlag(CLIENT *Client, CLIENT *Prefix, void *Buffer)
+{
+	IRC_WriteStrClientPrefix(Client, Prefix, "%s", Buffer);
+} /* cb_writeStrServersPrefixFlag */
+
+
+static bool
+Send_Marked_Connections(CLIENT *Prefix, const char *Buffer)
+{
+	CONN_ID conn;
+	bool ok = CONNECTED;
+
+	assert(Prefix != NULL);
+	assert(Buffer != NULL);
+
+	conn = Conn_First();
+	while (conn != NONE) {
+		if (Conn_Flag(conn) == SEND_TO_SERVER)
+			ok = Conn_WriteStr(conn, ":%s %s",
+					   Client_ID(Prefix), Buffer);
+		else if (Conn_Flag(conn) == SEND_TO_USER)
+			ok = Conn_WriteStr(conn, ":%s %s",
+					   Client_MaskCloaked(Prefix), Buffer);
+		if (!ok)
+			break;
+		conn = Conn_Next( conn );
+	}
+	return ok;
+}
+
+
 /* -eof- */
