@@ -1,20 +1,20 @@
 /*
  * ngIRCd -- The Next Generation IRC Daemon
- * Copyright (c)2001-2005 Alexander Barton (alex@barton.de)
+ * Copyright (c)2001-2012 Alexander Barton (alex@barton.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  * Please read the file COPYING, README and AUTHORS for more information.
- *
- * Logging functions
  */
 
-
 #include "portab.h"
 
-static char UNUSED id[] = "$Id: log.c,v 1.62 2006/08/05 09:16:21 fw Exp $";
+/**
+ * @file
+ * Logging functions
+ */
 
 #include "imp.h"
 #include <assert.h>
@@ -36,128 +36,83 @@ static char UNUSED id[] = "$Id: log.c,v 1.62 2006/08/05 09:16:21 fw Exp $";
 #include "ngircd.h"
 #include "defines.h"
 #include "conn.h"
-#include "client.h"
 #include "channel.h"
 #include "irc-write.h"
+#include "conf.h"
 
 #include "exp.h"
 #include "log.h"
 
 
-static char Init_Txt[127];
 static bool Is_Daemon;
 
-#ifdef DEBUG
-static char Error_File[FNAME_LEN];
+
+static void
+Log_Message(int Level, const char *msg)
+{
+	if (!Is_Daemon) {
+		/* log to console */
+		fprintf(stdout, "[%ld:%d %4ld] %s\n", (long)getpid(), Level,
+				(long)time(NULL) - NGIRCd_Start, msg);
+		fflush(stdout);
+	}
+#ifdef SYSLOG
+	else {
+		syslog(Level, "%s", msg);
+	}
 #endif
+}
 
 
-static void Wall_ServerNotice PARAMS(( char *Msg ));
-
-
+/**
+ * Initialitze logging.
+ * This function is called before the configuration file is read in.
+ *
+ * @param Daemon_Mode Set to true if ngIRCd is running as daemon.
+ */
 GLOBAL void
-Log_Init( bool Daemon_Mode )
+Log_Init(bool Daemon_Mode)
 {
 	Is_Daemon = Daemon_Mode;
-	
+
 #ifdef SYSLOG
-#ifndef LOG_CONS	/* Kludge: mips-dec-ultrix4.5 has no LOG_CONS/LOG_LOCAL5 */
+#ifndef LOG_CONS     /* Kludge: mips-dec-ultrix4.5 has no LOG_CONS */
 #define LOG_CONS 0
 #endif
-#ifndef LOG_LOCAL5
-#define LOG_LOCAL5 0
+#ifdef LOG_DAEMON
+	openlog(PACKAGE_NAME, LOG_CONS|LOG_PID, LOG_DAEMON);
+#else
+	openlog(PACKAGE_NAME, LOG_CONS|LOG_PID, 0);
 #endif
-	/* Syslog initialisieren */
-	openlog( PACKAGE_NAME, LOG_CONS|LOG_PID, LOG_LOCAL5 );
-#endif
-
-	/* Hello World! */
-	Log( LOG_NOTICE, "%s started.", NGIRCd_Version );
-	  
-	/* Informationen uebern den "Operation Mode" */
-	Init_Txt[0] = '\0';
-#ifdef DEBUG
-	if( NGIRCd_Debug )
-	{
-		strlcpy( Init_Txt, "debug-mode", sizeof Init_Txt );
-	}
-#endif
-	if( ! Is_Daemon )
-	{
-		if( Init_Txt[0] ) strlcat( Init_Txt, ", ", sizeof Init_Txt );
-		strlcat( Init_Txt, "no-daemon-mode", sizeof Init_Txt );
-	}
-	if( NGIRCd_Passive )
-	{
-		if( Init_Txt[0] ) strlcat( Init_Txt, ", ", sizeof Init_Txt );
-		strlcat( Init_Txt, "passive-mode", sizeof Init_Txt );
-	}
-#ifdef SNIFFER
-	if( NGIRCd_Sniffer )
-	{
-		if( Init_Txt[0] ) strlcat( Init_Txt, ", ", sizeof Init_Txt );
-		strlcat( Init_Txt, "network sniffer", sizeof Init_Txt );
-	}
-#endif
-	if( Init_Txt[0] ) Log( LOG_INFO, "Activating: %s.", Init_Txt );
-
-#ifdef DEBUG
-	Error_File[0] = '\0';
 #endif
 } /* Log_Init */
 
 
-#ifdef DEBUG
-
+/**
+ * Re-init logging after reading the configuration file.
+ */
 GLOBAL void
-Log_InitErrorfile( void )
+Log_ReInit(void)
 {
-	/* "Error-Log" initialisieren: stderr in Datei umlenken. Dort
-	 * landen z.B. alle Ausgaben von assert()-Aufrufen. */
-
-	/* Dateiname zusammen bauen */
-	snprintf( Error_File, sizeof Error_File, "%s/%s-%ld.err", ERROR_DIR, PACKAGE_NAME, (long)getpid( ));
-
-	/* stderr umlenken */
-	fflush( stderr );
-	if( ! freopen( Error_File, "w", stderr ))
-	{
-		Log( LOG_ERR, "Can't reopen stderr (\"%s\"): %s", Error_File, strerror( errno ));
-		return;
-	}
-
-	/* Einige Infos in das Error-File schreiben */
-	fputs( ctime( &NGIRCd_Start ), stderr );
-	fprintf( stderr, "%s started.\n", NGIRCd_Version );
-	fprintf( stderr, "Activating: %s\n\n", Init_Txt[0] ? Init_Txt : "-" );
-	fflush( stderr );
-
-#ifdef DEBUG
-	Log( LOG_DEBUG, "Redirected stderr to \"%s\".", Error_File );
+#ifdef SYSLOG
+#ifndef LOG_CONS     /* Kludge: mips-dec-ultrix4.5 has no LOG_CONS */
+#define LOG_CONS 0
 #endif
-} /* Log_InitErrfile */
-
+	closelog();
+	openlog(PACKAGE_NAME, LOG_CONS|LOG_PID, Conf_SyslogFacility);
 #endif
+	Log(LOG_NOTICE, "%s started.", NGIRCd_Version);
+	Log(LOG_INFO, "Using configuration file \"%s\" ...", NGIRCd_ConfFile);
+}
 
 
 GLOBAL void
 Log_Exit( void )
 {
-	/* Good Bye! */
-	if( NGIRCd_SignalRestart ) Log( LOG_NOTICE, "%s done (restarting).", PACKAGE_NAME );
-	else Log( LOG_NOTICE, "%s done.", PACKAGE_NAME );
-
-#ifdef DEBUG
-	if( Error_File[0] )
-	{
-		/* Error-File (stderr) loeschen */
-		if( unlink( Error_File ) != 0 ) Log( LOG_ERR, "Can't delete \"%s\": %s", Error_File, strerror( errno ));
-	}
-#endif
-
+	Log(LOG_NOTICE, "%s done%s, served %lu connections.", PACKAGE_NAME,
+	    NGIRCd_SignalRestart ? " (restarting)" : "", Conn_CountAccepted());
 #ifdef SYSLOG
-	/* syslog abmelden */
-	closelog( );
+	closelog();
 #endif
 } /* Log_Exit */
 
@@ -202,6 +157,8 @@ va_dcl
  * Logging function of ngIRCd.
  * This function logs messages to the console and/or syslog, whichever is
  * suitable for the mode ngIRCd is running in (daemon vs. non-daemon).
+ * If LOG_snotice is set, the log messages goes to all user with the mode +s
+ * set and the local &SERVER channel, too.
  * Please note: you sould use LogDebug(...) for debug messages!
  * @param Level syslog level (LOG_xxx)
  * @param Format Format string like printf().
@@ -218,7 +175,6 @@ const char *Format;
 va_dcl
 #endif
 {
-	/* Eintrag in Logfile(s) schreiben */
 	char msg[MAX_LOG_MSG_LEN];
 	bool snotice;
 	va_list ap;
@@ -239,7 +195,6 @@ va_dcl
 	if( Level == LOG_DEBUG ) return;
 #endif
 
-	/* String mit variablen Argumenten zusammenbauen ... */
 #ifdef PROTOTYPES
 	va_start( ap, Format );
 #else
@@ -248,124 +203,115 @@ va_dcl
 	vsnprintf( msg, MAX_LOG_MSG_LEN, Format, ap );
 	va_end( ap );
 
-	if( ! Is_Daemon )
-	{
-		/* auf Konsole ausgeben */
-		fprintf( stdout, "[%d:%d] %s\n", (int)getpid( ), Level, msg );
-		fflush( stdout );
-	}
-#ifdef SYSLOG
-	else
-	{
-		/* Syslog */
-		syslog( Level, "%s", msg );
-	}
-#endif
+	Log_Message(Level, msg);
 
-	if( Level <= LOG_CRIT )
-	{
-		/* log critical messages to stderr */
-		fprintf( stderr, "%s\n", msg );
-		fflush( stderr );
-	}
-
-	if( snotice )
-	{
-		/* NOTICE an lokale User mit "s"-Mode */
-		Wall_ServerNotice( msg );
+	if (snotice) {
+		/* Send NOTICE to all local users with mode +s and to the
+		 * local &SERVER channel */
+		Log_ServerNotice('s', "%s", msg);
+		Channel_LogServer(msg);
 	}
 } /* Log */
 
 
 GLOBAL void
-Log_Init_Resolver( void )
+Log_Init_Subprocess(char UNUSED *Name)
 {
 #ifdef SYSLOG
-	openlog( PACKAGE_NAME, LOG_CONS|LOG_PID, LOG_LOCAL5 );
+	openlog(PACKAGE_NAME, LOG_CONS|LOG_PID, Conf_SyslogFacility);
 #endif
 #ifdef DEBUG
-	Log_Resolver( LOG_DEBUG, "Resolver sub-process starting, PID %d.", getpid( ));
+	Log_Subprocess(LOG_DEBUG, "%s sub-process starting, PID %ld.",
+		     Name, (long)getpid());
 #endif
-} /* Log_Init_Resolver */
+}
 
 
 GLOBAL void
-Log_Exit_Resolver( void )
+Log_Exit_Subprocess(char UNUSED *Name)
 {
 #ifdef DEBUG
-	Log_Resolver( LOG_DEBUG, "Resolver sub-process %d done.", getpid( ));
+	Log_Subprocess(LOG_DEBUG, "%s sub-process %ld done.",
+		     Name, (long)getpid());
 #endif
 #ifdef SYSLOG
 	closelog( );
 #endif
-} /* Log_Exit_Resolver */
+}
 
 
 #ifdef PROTOTYPES
 GLOBAL void
-Log_Resolver( const int Level, const char *Format, ... )
+Log_Subprocess(const int Level, const char *Format, ...)
 #else
 GLOBAL void
-Log_Resolver( Level, Format, va_alist )
+Log_Subprocess(Level, Format, va_alist)
 const int Level;
 const char *Format;
 va_dcl
 #endif
 {
-	/* Eintrag des Resolver in Logfile(s) schreiben */
-
 	char msg[MAX_LOG_MSG_LEN];
 	va_list ap;
 
-	assert( Format != NULL );
+	assert(Format != NULL);
 
 #ifdef DEBUG
-	if(( Level == LOG_DEBUG ) && ( ! NGIRCd_Debug )) return;
+	if ((Level == LOG_DEBUG) && (!NGIRCd_Debug))
+		return;
 #else
-	if( Level == LOG_DEBUG ) return;
+	if (Level == LOG_DEBUG)
+		return;
 #endif
 
-	/* String mit variablen Argumenten zusammenbauen ... */
 #ifdef PROTOTYPES
-	va_start( ap, Format );
+	va_start(ap, Format);
 #else
-	va_start( ap );
+	va_start(ap);
 #endif
-	vsnprintf( msg, MAX_LOG_MSG_LEN, Format, ap );
-	va_end( ap );
+	vsnprintf(msg, MAX_LOG_MSG_LEN, Format, ap);
+	va_end(ap);
 
-	if( ! Is_Daemon )
-	{
-		/* Output to console */
-		fprintf( stdout, "[%d:%d] %s\n", (int)getpid( ), Level, msg );
-		fflush( stdout );
-	}
-#ifdef SYSLOG
-	else syslog( Level, "%s", msg );
-#endif
-} /* Log_Resolver */
+	Log_Message(Level, msg);
+}
 
 
 /**
- * Send log messages to users flagged with the "s" mode.
- * @param Msg The message to send.
+ * Send a log message to all local users flagged with the given user mode.
+ * @param UserMode User mode which the target user must have set,
+ * @param Format The format string.
  */
-static void
-Wall_ServerNotice( char *Msg )
+#ifdef PROTOTYPES
+GLOBAL void
+Log_ServerNotice(const char UserMode, const char *Format, ... )
+#else
+GLOBAL void
+Log_ServerNotice(UserMode, Format, va_alist)
+const char UserMode;
+const char *Format;
+va_dcl
+#endif
 {
 	CLIENT *c;
+	char msg[MAX_LOG_MSG_LEN];
+	va_list ap;
 
-	assert( Msg != NULL );
+	assert(Format != NULL);
 
-	c = Client_First( );
-	while(c) {
-		if (Client_Conn(c) > NONE && Client_HasMode(c, 's'))
+#ifdef PROTOTYPES
+	va_start(ap, Format);
+#else
+	va_start(ap);
+#endif
+	vsnprintf(msg, MAX_LOG_MSG_LEN, Format, ap);
+	va_end(ap);
+
+	for(c=Client_First(); c != NULL; c=Client_Next(c)) {
+		if (Client_Conn(c) > NONE && Client_HasMode(c, UserMode))
 			IRC_WriteStrClient(c, "NOTICE %s :%s%s", Client_ID(c),
-							NOTICE_TXTPREFIX, Msg);
-
-		c = Client_Next( c );
+							NOTICE_TXTPREFIX, msg);
 	}
-} /* Wall_ServerNotice */
+} /* Log_ServerNotice */
 
 
 /* -eof- */
