@@ -1,6 +1,6 @@
 /*
  * ngIRCd -- The Next Generation IRC Daemon
- * Copyright (c)2001-2008 Alexander Barton (alex@barton.de).
+ * Copyright (c)2001-2012 Alexander Barton (alex@barton.de) and Contributors.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -9,7 +9,6 @@
  * Please read the file COPYING, README and AUTHORS for more information.
  */
 
-
 #include "portab.h"
 
 /**
@@ -29,33 +28,31 @@
 #include <time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/wait.h>
 #include <fcntl.h>
 #include <pwd.h>
 #include <grp.h>
 
+#if defined(DEBUG) && defined(HAVE_MTRACE)
+#include <mcheck.h>
+#endif
+
 #include "defines.h"
-#include "resolve.h"
 #include "conn.h"
-#include "client.h"
+#include "class.h"
+#include "conf-ssl.h"
 #include "channel.h"
 #include "conf.h"
 #include "lists.h"
 #include "log.h"
 #include "parse.h"
+#include "sighandlers.h"
+#include "io.h"
 #include "irc.h"
 
-#ifdef ZEROCONF
-#include "rendezvous.h"
-#endif
-
 #include "exp.h"
 #include "ngircd.h"
 
 
-static void Initialize_Signal_Handler PARAMS(( void ));
-static void Signal_Handler PARAMS(( int Signal ));
-
 static void Show_Version PARAMS(( void ));
 static void Show_Help PARAMS(( void ));
 
@@ -64,29 +61,40 @@ static void Pidfile_Delete PARAMS(( void ));
 
 static void Fill_Version PARAMS(( void ));
 
-static void Setup_FDStreams PARAMS(( void ));
+static void Random_Init PARAMS(( void ));
+
+static void Setup_FDStreams PARAMS(( int fd ));
 
 static bool NGIRCd_Init PARAMS(( bool ));
 
+
 /**
  * The main() function of ngIRCd.
+ *
  * Here all starts: this function is called by the operating system loader,
  * it is the first portion of code executed of ngIRCd.
+ *
  * @param argc The number of arguments passed to ngIRCd on the command line.
  * @param argv An array containing all the arguments passed to ngIRCd.
  * @return Global exit code of ngIRCd, zero on success.
  */
 GLOBAL int
-main( int argc, const char *argv[] )
+main(int argc, const char *argv[])
 {
 	bool ok, configtest = false;
 	bool NGIRCd_NoDaemon = false;
 	int i;
 	size_t n;
 
-	umask( 0077 );
+#if defined(DEBUG) && defined(HAVE_MTRACE)
+	/* enable GNU libc memory tracing when running in debug mode
+	 * and functionality available */
+	mtrace();
+#endif
 
-	NGIRCd_SignalQuit = NGIRCd_SignalRestart = NGIRCd_SignalRehash = false;
+	umask(0077);
+
+	NGIRCd_SignalQuit = NGIRCd_SignalRestart = false;
 	NGIRCd_Passive = false;
 #ifdef DEBUG
 	NGIRCd_Debug = false;
@@ -94,76 +102,62 @@ main( int argc, const char *argv[] )
 #ifdef SNIFFER
 	NGIRCd_Sniffer = false;
 #endif
-	strlcpy( NGIRCd_ConfFile, SYSCONFDIR, sizeof( NGIRCd_ConfFile ));
-	strlcat( NGIRCd_ConfFile, CONFIG_FILE, sizeof( NGIRCd_ConfFile ));
+	strlcpy(NGIRCd_ConfFile, SYSCONFDIR, sizeof(NGIRCd_ConfFile));
+	strlcat(NGIRCd_ConfFile, CONFIG_FILE, sizeof(NGIRCd_ConfFile));
 
-	Fill_Version( );
+	Fill_Version();
 
-	/* Kommandozeile parsen */
-	for( i = 1; i < argc; i++ )
-	{
+	/* parse conmmand line */
+	for (i = 1; i < argc; i++) {
 		ok = false;
-		if(( argv[i][0] == '-' ) && ( argv[i][1] == '-' ))
-		{
-			/* Lange Option */
-
-			if( strcmp( argv[i], "--config" ) == 0 )
-			{
-				if( i + 1 < argc )
-				{
+		if (argv[i][0] == '-' && argv[i][1] == '-') {
+			/* long option */
+			if (strcmp(argv[i], "--config") == 0) {
+				if (i + 1 < argc) {
 					/* Ok, there's an parameter left */
-					strlcpy( NGIRCd_ConfFile, argv[i + 1], sizeof( NGIRCd_ConfFile ));
-
+					strlcpy(NGIRCd_ConfFile, argv[i+1],
+						sizeof(NGIRCd_ConfFile));
 					/* next parameter */
 					i++; ok = true;
 				}
 			}
-			if( strcmp( argv[i], "--configtest" ) == 0 )
-			{
+			if (strcmp(argv[i], "--configtest") == 0) {
 				configtest = true;
 				ok = true;
 			}
 #ifdef DEBUG
-			if( strcmp( argv[i], "--debug" ) == 0 )
-			{
+			if (strcmp(argv[i], "--debug") == 0) {
 				NGIRCd_Debug = true;
 				ok = true;
 			}
 #endif
-			if( strcmp( argv[i], "--help" ) == 0 )
-			{
-				Show_Version( );
-				puts( "" ); Show_Help( ); puts( "" );
-				exit( 1 );
+			if (strcmp(argv[i], "--help") == 0) {
+				Show_Version();
+				puts(""); Show_Help( ); puts( "" );
+				exit(1);
 			}
-			if( strcmp( argv[i], "--nodaemon" ) == 0 )
-			{
+			if (strcmp(argv[i], "--nodaemon") == 0) {
 				NGIRCd_NoDaemon = true;
 				ok = true;
 			}
-			if( strcmp( argv[i], "--passive" ) == 0 )
-			{
+			if (strcmp(argv[i], "--passive") == 0) {
 				NGIRCd_Passive = true;
 				ok = true;
 			}
 #ifdef SNIFFER
-			if( strcmp( argv[i], "--sniffer" ) == 0 )
-			{
+			if (strcmp(argv[i], "--sniffer") == 0) {
 				NGIRCd_Sniffer = true;
 				ok = true;
 			}
 #endif
-			if( strcmp( argv[i], "--version" ) == 0 )
-			{
-				Show_Version( );
-				exit( 1 );
+			if (strcmp(argv[i], "--version") == 0) {
+				Show_Version();
+				exit(1);
 			}
 		}
-		else if(( argv[i][0] == '-' ) && ( argv[i][1] != '-' ))
-		{
-			/* Kurze Option */
-			for( n = 1; n < strlen( argv[i] ); n++ )
-			{
+		else if(argv[i][0] == '-' && argv[i][1] != '-') {
+			/* short option */
+			for (n = 1; n < strlen(argv[i]); n++) {
 				ok = false;
 #ifdef DEBUG
 				if (argv[i][n] == 'd') {
@@ -172,14 +166,14 @@ main( int argc, const char *argv[] )
 				}
 #endif
 				if (argv[i][n] == 'f') {
-					if(( ! argv[i][n + 1] ) && ( i + 1 < argc ))
-					{
+					if (!argv[i][n+1] && i+1 < argc) {
 						/* Ok, next character is a blank */
-						strlcpy( NGIRCd_ConfFile, argv[i + 1], sizeof( NGIRCd_ConfFile ));
+						strlcpy(NGIRCd_ConfFile, argv[i+1],
+							sizeof(NGIRCd_ConfFile));
 
 						/* go to the following parameter */
 						i++;
-						n = strlen( argv[i] );
+						n = strlen(argv[i]);
 						ok = true;
 					}
 				}
@@ -214,55 +208,58 @@ main( int argc, const char *argv[] )
 					exit(1);
 				}
 
-				if (! ok) {
-					printf( "%s: invalid option \"-%c\"!\n", PACKAGE_NAME, argv[i][n] );
-					printf( "Try \"%s --help\" for more information.\n", PACKAGE_NAME );
-					exit( 1 );
+				if (!ok) {
+					printf("%s: invalid option \"-%c\"!\n",
+					       PACKAGE_NAME, argv[i][n]);
+					printf("Try \"%s --help\" for more information.\n",
+					       PACKAGE_NAME);
+					exit(1);
 				}
 			}
 
 		}
-		if( ! ok )
-		{
-			printf( "%s: invalid option \"%s\"!\n", PACKAGE_NAME, argv[i] );
-			printf( "Try \"%s --help\" for more information.\n", PACKAGE_NAME );
-			exit( 1 );
+		if (!ok) {
+			printf("%s: invalid option \"%s\"!\n",
+			       PACKAGE_NAME, argv[i]);
+			printf("Try \"%s --help\" for more information.\n",
+			       PACKAGE_NAME);
+			exit(1);
 		}
 	}
 
-	/* Debug-Level (fuer IRC-Befehl "VERSION") ermitteln */
+	/* Debug level for "VERSION" command */
 	NGIRCd_DebugLevel[0] = '\0';
 #ifdef DEBUG
-	if( NGIRCd_Debug ) strcpy( NGIRCd_DebugLevel, "1" );
+	if (NGIRCd_Debug)
+		strcpy(NGIRCd_DebugLevel, "1");
 #endif
 #ifdef SNIFFER
-	if( NGIRCd_Sniffer )
-	{
+	if (NGIRCd_Sniffer) {
 		NGIRCd_Debug = true;
-		strcpy( NGIRCd_DebugLevel, "2" );
+		strcpy(NGIRCd_DebugLevel, "2");
 	}
 #endif
 
-	/* Soll nur die Konfigurations ueberprueft und ausgegeben werden? */
-	if( configtest )
-	{
-		Show_Version( ); puts( "" );
-		exit( Conf_Test( ));
+	if (configtest) {
+		Show_Version(); puts("");
+		exit(Conf_Test());
 	}
-	
-	while( ! NGIRCd_SignalQuit )
-	{
-		/* Initialize global variables */
-		NGIRCd_Start = time( NULL );
-		(void)strftime( NGIRCd_StartStr, 64, "%a %b %d %Y at %H:%M:%S (%Z)", localtime( &NGIRCd_Start ));
 
-		NGIRCd_SignalRehash = false;
+	while (!NGIRCd_SignalQuit) {
+		/* Initialize global variables */
+		NGIRCd_Start = time(NULL);
+		(void)strftime(NGIRCd_StartStr, 64,
+			       "%a %b %d %Y at %H:%M:%S (%Z)",
+			       localtime(&NGIRCd_Start));
+
 		NGIRCd_SignalRestart = false;
 		NGIRCd_SignalQuit = false;
 
 		/* Initialize modules, part I */
-		Log_Init( ! NGIRCd_NoDaemon );
-		Conf_Init( );
+		Log_Init(!NGIRCd_NoDaemon);
+		Random_Init();
+		Conf_Init();
+		Log_ReInit();
 
 		/* Initialize the "main program": chroot environment, user and
 		 * group ID, ... */
@@ -273,75 +270,80 @@ main( int argc, const char *argv[] )
 
 		/* Initialize modules, part II: these functions are eventually
 		 * called with already dropped privileges ... */
-		Channel_Init( );
-		Client_Init( );
-#ifdef ZEROCONF
-		Rendezvous_Init( );
-#endif
-		Conn_Init( );
+		Channel_Init();
+		Client_Init();
+		Conn_Init();
+		Class_Init();
 
-#ifdef DEBUG
-		/* Redirect stderr handle to "error file" for debugging
-		 * when not running in "no daemon" mode: */
-		if( ! NGIRCd_NoDaemon ) Log_InitErrorfile( );
-#endif
-
-		/* Signal-Handler initialisieren */
-		Initialize_Signal_Handler( );
-
-		/* Protokoll- und Server-Identifikation erzeugen. Die vom ngIRCd
-		 * beim PASS-Befehl verwendete Syntax sowie die erweiterten Flags
-		 * sind in doc/Protocol.txt beschrieben. */
-#ifdef IRCPLUS
-		snprintf( NGIRCd_ProtoID, sizeof NGIRCd_ProtoID, "%s%s %s|%s:%s", PROTOVER, PROTOIRCPLUS, PACKAGE_NAME, PACKAGE_VERSION, IRCPLUSFLAGS );
-#ifdef ZLIB
-		strcat( NGIRCd_ProtoID, "Z" );
-#endif
-		if( Conf_OperCanMode ) strcat( NGIRCd_ProtoID, "o" );
-#else
-		snprintf( NGIRCd_ProtoID, sizeof NGIRCd_ProtoID, "%s%s %s|%s", PROTOVER, PROTOIRC, PACKAGE_NAME, PACKAGE_VERSION );
-#endif
-		strlcat( NGIRCd_ProtoID, " P", sizeof NGIRCd_ProtoID );
-#ifdef ZLIB
-		strlcat( NGIRCd_ProtoID, "Z", sizeof NGIRCd_ProtoID );
-#endif
-		Log( LOG_DEBUG, "Protocol and server ID is \"%s\".", NGIRCd_ProtoID );
-
-		/* Vordefinierte Channels anlegen */
-		Channel_InitPredefined( );
-
-		/* Listen-Ports initialisieren */
-		if( Conn_InitListeners( ) < 1 )
-		{
-			Log( LOG_ALERT, "Server isn't listening on a single port!" );
-			Log( LOG_ALERT, "%s exiting due to fatal errors!", PACKAGE_NAME );
-			Pidfile_Delete( );
-			exit( 1 );
+		if (!io_library_init(CONNECTION_POOL)) {
+			Log(LOG_ALERT,
+			    "Fatal: Could not initialize IO routines: %s",
+			    strerror(errno));
+			exit(1);
 		}
-		
-		/* Hauptschleife */
-		Conn_Handler( );
 
-		/* Alles abmelden */
-		Conn_Exit( );
-#ifdef ZEROCONF
-		Rendezvous_Exit( );
+		if (!Signals_Init()) {
+			Log(LOG_ALERT,
+			    "Fatal: Could not set up signal handlers: %s",
+			    strerror(errno));
+			exit(1);
+		}
+
+		/* Create protocol and server identification. The syntax
+		 * used by ngIRCd in PASS commands and the known "extended
+		 * flags" are described in doc/Protocol.txt. */
+#ifdef IRCPLUS
+		snprintf(NGIRCd_ProtoID, sizeof NGIRCd_ProtoID, "%s%s %s|%s:%s",
+			 PROTOVER, PROTOIRCPLUS, PACKAGE_NAME, PACKAGE_VERSION,
+			 IRCPLUSFLAGS);
+#ifdef ZLIB
+		strcat(NGIRCd_ProtoID, "Z");
 #endif
-		Client_Exit( );
-		Channel_Exit( );
-		Log_Exit( );
+		if (Conf_OperCanMode)
+			strcat(NGIRCd_ProtoID, "o");
+#else /* IRCPLUS */
+		snprintf(NGIRCd_ProtoID, sizeof NGIRCd_ProtoID, "%s%s %s|%s",
+			 PROTOVER, PROTOIRC, PACKAGE_NAME, PACKAGE_VERSION);
+#endif /* IRCPLUS */
+		strlcat(NGIRCd_ProtoID, " P", sizeof NGIRCd_ProtoID);
+#ifdef ZLIB
+		strlcat(NGIRCd_ProtoID, "Z", sizeof NGIRCd_ProtoID);
+#endif
+		LogDebug("Protocol and server ID is \"%s\".", NGIRCd_ProtoID);
+
+		Channel_InitPredefined();
+
+		if (Conn_InitListeners() < 1) {
+			Log(LOG_ALERT,
+			    "Server isn't listening on a single port!" );
+			Log(LOG_ALERT,
+			    "%s exiting due to fatal errors!", PACKAGE_NAME);
+			Pidfile_Delete();
+			exit(1);
+		}
+
+		/* Main Run Loop */
+		Conn_Handler();
+
+		Conn_Exit();
+		Client_Exit();
+		Channel_Exit();
+		Class_Exit();
+		Log_Exit();
 	}
-	Pidfile_Delete( );
+	Pidfile_Delete();
 
 	return 0;
 } /* main */
 
 
 /**
- * Generate ngIRCd "version string".
- * This string is generated once and then stored in NGIRCd_Version for
- * further usage, for example by the IRC command VERSION and the --version
- * command line switch.
+ * Generate ngIRCd "version strings".
+ *
+ * The ngIRCd version information is generated once and then stored in the
+ * NGIRCd_Version and NGIRCd_VersionAddition string variables for further
+ * usage, for example by the IRC command "VERSION" and the --version command
+ * line switch.
  */
 static void
 Fill_Version( void )
@@ -354,55 +356,50 @@ Fill_Version( void )
 #ifdef ZLIB
 	if( NGIRCd_VersionAddition[0] )
 		strlcat( NGIRCd_VersionAddition, "+", sizeof NGIRCd_VersionAddition );
-
 	strlcat( NGIRCd_VersionAddition, "ZLIB", sizeof NGIRCd_VersionAddition );
 #endif
+#ifdef SSL_SUPPORT
+	if ( NGIRCd_VersionAddition[0] ) strlcat( NGIRCd_VersionAddition, "+", sizeof NGIRCd_VersionAddition );
+	strlcat( NGIRCd_VersionAddition, "SSL", sizeof NGIRCd_VersionAddition );
+#endif
 #ifdef TCPWRAP
 	if( NGIRCd_VersionAddition[0] )
 			strlcat( NGIRCd_VersionAddition, "+", sizeof NGIRCd_VersionAddition );
-
 	strlcat( NGIRCd_VersionAddition, "TCPWRAP", sizeof NGIRCd_VersionAddition );
 #endif
-#ifdef ZEROCONF
-	if( NGIRCd_VersionAddition[0] )
-		strlcat( NGIRCd_VersionAddition, "+", sizeof NGIRCd_VersionAddition );
-
-	strlcat( NGIRCd_VersionAddition, "ZEROCONF", sizeof NGIRCd_VersionAddition );
-#endif
 #ifdef IDENTAUTH
 	if( NGIRCd_VersionAddition[0] )
 		strlcat( NGIRCd_VersionAddition, "+", sizeof NGIRCd_VersionAddition );
-
 	strlcat( NGIRCd_VersionAddition, "IDENT", sizeof NGIRCd_VersionAddition );
 #endif
+#ifdef PAM
+	if (NGIRCd_VersionAddition[0])
+		strlcat(NGIRCd_VersionAddition, "+", sizeof NGIRCd_VersionAddition);
+	strlcat(NGIRCd_VersionAddition, "PAM", sizeof NGIRCd_VersionAddition);
+#endif
 #ifdef DEBUG
 	if( NGIRCd_VersionAddition[0] )
 		strlcat( NGIRCd_VersionAddition, "+", sizeof NGIRCd_VersionAddition );
-
 	strlcat( NGIRCd_VersionAddition, "DEBUG", sizeof NGIRCd_VersionAddition );
 #endif
 #ifdef SNIFFER
 	if( NGIRCd_VersionAddition[0] )
 		strlcat( NGIRCd_VersionAddition, "+", sizeof NGIRCd_VersionAddition );
-
 	strlcat( NGIRCd_VersionAddition, "SNIFFER", sizeof NGIRCd_VersionAddition );
 #endif
 #ifdef STRICT_RFC
 	if( NGIRCd_VersionAddition[0] )
 		strlcat( NGIRCd_VersionAddition, "+", sizeof NGIRCd_VersionAddition );
-
 	strlcat( NGIRCd_VersionAddition, "RFC", sizeof NGIRCd_VersionAddition );
 #endif
 #ifdef IRCPLUS
 	if( NGIRCd_VersionAddition[0] )
 		strlcat( NGIRCd_VersionAddition, "+", sizeof NGIRCd_VersionAddition );
-
 	strlcat( NGIRCd_VersionAddition, "IRCPLUS", sizeof NGIRCd_VersionAddition );
 #endif
 #ifdef WANT_IPV6
 	if (NGIRCd_VersionAddition[0])
 		strlcat(NGIRCd_VersionAddition, "+", sizeof(NGIRCd_VersionAddition));
-
 	strlcat(NGIRCd_VersionAddition, "IPv6", sizeof(NGIRCd_VersionAddition));
 #endif
 	if( NGIRCd_VersionAddition[0] )
@@ -416,138 +413,7 @@ Fill_Version( void )
 
 	snprintf(NGIRCd_Version, sizeof NGIRCd_Version, "%s %s-%s",
 		 PACKAGE_NAME, PACKAGE_VERSION, NGIRCd_VersionAddition);
-	} /* Fill_Version */
-
-
-/**
- * Reload the server configuration file.
- */
-GLOBAL void
-NGIRCd_Rehash( void )
-{
-	char old_name[CLIENT_ID_LEN];
-	unsigned old_nicklen;
-
-	Log( LOG_NOTICE|LOG_snotice, "Re-reading configuration NOW!" );
-	NGIRCd_SignalRehash = false;
-
-	/* Remember old server name and nick name length */
-	strlcpy( old_name, Conf_ServerName, sizeof old_name );
-	old_nicklen = Conf_MaxNickLength;
-
-	/* Re-read configuration ... */
-	if (!Conf_Rehash( ))
-		return;
-
-	/* Close down all listening sockets */
-	Conn_ExitListeners( );
-
-	/* Recover old server name and nick name length: these values can't
-	 * be changed during run-time */
-	if (strcmp(old_name, Conf_ServerName) != 0 ) {
-		strlcpy(Conf_ServerName, old_name, sizeof Conf_ServerName);
-		Log(LOG_ERR, "Can't change \"ServerName\" on runtime! Ignored new name.");
-	}
-	if (old_nicklen != Conf_MaxNickLength) {
-		Conf_MaxNickLength = old_nicklen;
-		Log(LOG_ERR, "Can't change \"MaxNickLength\" on runtime! Ignored new value.");
-	}
-
-	/* Create new pre-defined channels */
-	Channel_InitPredefined( );
-	
-	/* Start listening on sockets */
-	Conn_InitListeners( );
-
-	/* Sync configuration with established connections */
-	Conn_SyncServerStruct( );
-
-	Log( LOG_NOTICE|LOG_snotice, "Re-reading of configuration done." );
-} /* NGIRCd_Rehash */
-
-
-/**
- * Initialize the signal handler.
- */
-static void
-Initialize_Signal_Handler( void )
-{
-	/* Signal-Handler initialisieren: einige Signale
-	 * werden ignoriert, andere speziell behandelt. */
-
-#ifdef HAVE_SIGACTION
-	/* sigaction() ist vorhanden */
-
-	struct sigaction saction;
-
-	/* Signal-Struktur initialisieren */
-	memset( &saction, 0, sizeof( saction ));
-	saction.sa_handler = Signal_Handler;
-#ifdef SA_RESTART
-	saction.sa_flags |= SA_RESTART;
-#endif
-#ifdef SA_NOCLDWAIT
-	saction.sa_flags |= SA_NOCLDWAIT;
-#endif
-
-	/* Signal-Handler einhaengen */
-	sigaction(SIGINT, &saction, NULL);
-	sigaction(SIGQUIT, &saction, NULL);
-	sigaction(SIGTERM, &saction, NULL);
-	sigaction(SIGHUP, &saction, NULL);
-	sigaction(SIGCHLD, &saction, NULL);
-
-	/* einige Signale ignorieren */
-	saction.sa_handler = SIG_IGN;
-	sigaction(SIGPIPE, &saction, NULL);
-#else
-	/* kein sigaction() vorhanden */
-
-	/* Signal-Handler einhaengen */
-	signal(SIGINT, Signal_Handler);
-	signal(SIGQUIT, Signal_Handler);
-	signal(SIGTERM, Signal_Handler);
-	signal(SIGHUP, Signal_Handler);
-	signal(SIGCHLD, Signal_Handler);
-
-	/* einige Signale ignorieren */
-	signal(SIGPIPE, SIG_IGN);
-#endif
-} /* Initialize_Signal_Handler */
-
-
-/**
- * Signal handler of ngIRCd.
- * This function is called whenever ngIRCd catches a signal sent by the
- * user and/or the system to it. For example SIGTERM and SIGHUP.
- * @param Signal Number of the signal to handle.
- */
-static void
-Signal_Handler( int Signal )
-{
-	switch( Signal )
-	{
-		case SIGTERM:
-		case SIGINT:
-		case SIGQUIT:
-			/* wir soll(t)en uns wohl beenden ... */
-			NGIRCd_SignalQuit = true;
-			break;
-		case SIGHUP:
-			/* Konfiguration neu einlesen: */
-			NGIRCd_SignalRehash = true;
-			break;
-		case SIGCHLD:
-			/* Child-Prozess wurde beendet. Zombies vermeiden: */
-			while( waitpid( -1, NULL, WNOHANG ) > 0);
-			break;
-#ifdef DEBUG
-		default:
-			/* unbekanntes bzw. unbehandeltes Signal */
-			Log( LOG_DEBUG, "Got signal %d! Ignored.", Signal );
-#endif
-	}
-} /* Signal_Handler */
+} /* Fill_Version */
 
 
 /**
@@ -557,7 +423,7 @@ static void
 Show_Version( void )
 {
 	puts( NGIRCd_Version );
-	puts( "Copyright (c)2001-2008 Alexander Barton (<alex@barton.de>) and Contributors." );
+	puts( "Copyright (c)2001-2012 Alexander Barton (<alex@barton.de>) and Contributors." );
 	puts( "Homepage: <http://ngircd.barton.de/>\n" );
 	puts( "This is free software; see the source for copying conditions. There is NO" );
 	puts( "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE." );
@@ -607,7 +473,8 @@ Pidfile_Delete( void )
 
 /**
  * Create the file containing the process ID of ngIRCd ("PID file").
- * @param pid The process ID to be stored in this file.
+ *
+ * @param pid	The process ID to be stored in this file.
  */
 static void
 Pidfile_Create(pid_t pid)
@@ -631,7 +498,8 @@ Pidfile_Create(pid_t pid)
 
 	len = snprintf(pidbuf, sizeof pidbuf, "%ld\n", (long)pid);
 	if (len < 0 || len >= (int)sizeof pidbuf) {
-		Log( LOG_ERR, "Error converting pid");
+		Log(LOG_ERR, "Error converting pid");
+		close(pidfd);
 		return;
 	}
 	
@@ -645,110 +513,185 @@ Pidfile_Create(pid_t pid)
 
 /**
  * Redirect stdin, stdout and stderr to apropriate file handles.
+ *
+ * @param fd	The file handle stdin, stdout and stderr should be redirected to.
  */
 static void
-Setup_FDStreams( void )
+Setup_FDStreams(int fd)
 {
-	int fd;
-
-	/* Test if we can open /dev/null for reading and writing. If not
-	 * we are most probably chrooted already and the server has been
-	 * restarted. So we simply don't try to redirect stdXXX ... */
-	fd = open( "/dev/null", O_RDWR );
-	if ( fd < 0 ) {
-		Log(LOG_WARNING, "Could not open /dev/null: %s", strerror(errno));	
+	if (fd < 0)
 		return;
-	} 
 
 	fflush(stdout);
 	fflush(stderr);
 
 	/* Create new stdin(0), stdout(1) and stderr(2) descriptors */
 	dup2( fd, 0 ); dup2( fd, 1 ); dup2( fd, 2 );
-
-	/* Close newly opened file descriptor if not stdin/out/err */
-	if( fd > 2 ) close( fd );
 } /* Setup_FDStreams */
 
 
+/**
+ * Get user and group ID of unprivileged "nobody" user.
+ *
+ * @param uid	User ID
+ * @param gid	Group ID
+ * @return	true on success.
+ */
 static bool
 NGIRCd_getNobodyID(uid_t *uid, gid_t *gid )
 {
 	struct passwd *pwd;
 
-	pwd = getpwnam("nobody");
-	if (!pwd) return false;
+#ifdef __CYGWIN__
+	/* Cygwin kludge.
+	 * It can return EINVAL instead of EPERM
+	 * so, if we are already unprivileged,
+	 * use id of current user.
+	 */
+	if (geteuid() && getuid()) {
+		*uid = getuid();
+		*gid = getgid();
+		return true;
+	}
+#endif
 
-	if ( !pwd->pw_uid || !pwd->pw_gid)
+	pwd = getpwnam("nobody");
+	if (!pwd)
 		return false;
 
-	*uid = pwd->pw_uid;	
+	if (!pwd->pw_uid || !pwd->pw_gid)
+		return false;
+
+	*uid = pwd->pw_uid;
 	*gid = pwd->pw_gid;
 	endpwent();
 
-	return true;	
-}
+	return true;
+} /* NGIRCd_getNobodyID */
 
 
 static bool
-NGIRCd_Init( bool NGIRCd_NoDaemon ) 
+Random_Init_Kern(const char *file)
+{
+	unsigned int seed;
+	bool ret = false;
+	int fd = open(file, O_RDONLY);
+	if (fd >= 0) {
+		if (read(fd, &seed, sizeof(seed)) == sizeof(seed))
+			ret = true;
+		close(fd);
+		srand(seed);
+	}
+	return ret;
+}
+
+/**
+ * Initialize libc rand(3) number generator
+ */
+static void
+Random_Init(void)
+{
+	if (Random_Init_Kern("/dev/urandom"))
+		return;
+	if (Random_Init_Kern("/dev/random"))
+		return;
+	if (Random_Init_Kern("/dev/arandom"))
+		return;
+	srand(rand() ^ (unsigned)getpid() ^ (unsigned)time(NULL));
+}
+
+
+/**
+ * Initialize ngIRCd daemon.
+ *
+ * @param NGIRCd_NoDaemon Set to true if ngIRCd should run in the
+ *		foreground (and not as a daemon).
+ * @return true on success.
+ */
+static bool
+NGIRCd_Init(bool NGIRCd_NoDaemon)
 {
 	static bool initialized;
 	bool chrooted = false;
 	struct passwd *pwd;
 	struct group *grp;
-	int real_errno;
+	int real_errno, fd = -1;
 	pid_t pid;
 
 	if (initialized)
 		return true;
 
-	if( Conf_Chroot[0] ) {
-		if( chdir( Conf_Chroot ) != 0 ) {
-			Log( LOG_ERR, "Can't chdir() in ChrootDir (%s): %s", Conf_Chroot, strerror( errno ));
-			return false;
+	if (!NGIRCd_NoDaemon) {
+		/* open /dev/null before chroot() */
+		fd = open( "/dev/null", O_RDWR);
+		if (fd < 0)
+			Log(LOG_WARNING, "Could not open /dev/null: %s",
+			    strerror(errno));
+	}
+
+	/* SSL initialization */
+	if (!ConnSSL_InitLibrary())
+		Log(LOG_WARNING,
+		    "Warning: Error during SSL initialization, continuing ...");
+
+	/* Change root */
+	if (Conf_Chroot[0]) {
+		if (chdir(Conf_Chroot) != 0) {
+			Log(LOG_ERR, "Can't chdir() in ChrootDir (%s): %s",
+			    Conf_Chroot, strerror(errno));
+			goto out;
 		}
 
-		if( chroot( Conf_Chroot ) != 0 ) {
-			if (errno != EPERM) {
-				Log( LOG_ERR, "Can't change root directory to \"%s\": %s",
-								Conf_Chroot, strerror( errno ));
-
-				return false;
-			}
+		if (chroot(Conf_Chroot) != 0) {
+			Log(LOG_ERR,
+			    "Can't change root directory to \"%s\": %s",
+			    Conf_Chroot, strerror(errno));
+			goto out;
 		} else {
 			chrooted = true;
-			Log( LOG_INFO, "Changed root and working directory to \"%s\".", Conf_Chroot );
+			Log(LOG_INFO,
+			    "Changed root and working directory to \"%s\".",
+			    Conf_Chroot);
 		}
 	}
 
+	/* Check user ID */
 	if (Conf_UID == 0) {
-		Log(LOG_INFO, "ServerUID must not be 0, using \"nobody\" instead.", Conf_UID);
-
-  		if (! NGIRCd_getNobodyID(&Conf_UID, &Conf_GID)) {
-			Log(LOG_WARNING, "Could not get user/group ID of user \"nobody\": %s",
-					errno ? strerror(errno) : "not found" );
-			return false;
+		pwd = getpwuid(0);
+		Log(LOG_INFO,
+		    "ServerUID must not be %s(0), using \"nobody\" instead.",
+		    pwd ? pwd->pw_name : "?");
+		if (!NGIRCd_getNobodyID(&Conf_UID, &Conf_GID)) {
+			Log(LOG_WARNING,
+			    "Could not get user/group ID of user \"nobody\": %s",
+			    errno ? strerror(errno) : "not found" );
+			goto out;
 		}
 	}
 
+	/* Change group ID */
 	if (getgid() != Conf_GID) {
-		/* Change group ID */
 		if (setgid(Conf_GID) != 0) {
 			real_errno = errno;
-			Log( LOG_ERR, "Can't change group ID to %u: %s", Conf_GID, strerror( errno ));
+			grp = getgrgid(Conf_GID);
+			Log(LOG_ERR, "Can't change group ID to %s(%u): %s",
+			    grp ? grp->gr_name : "?", Conf_GID,
+			    strerror(errno));
 			if (real_errno != EPERM) 
-				return false;
+				goto out;
 		}
 	}
 
+	/* Change user ID */
 	if (getuid() != Conf_UID) {
-		/* Change user ID */
 		if (setuid(Conf_UID) != 0) {
 			real_errno = errno;
-			Log(LOG_ERR, "Can't change user ID to %u: %s", Conf_UID, strerror(errno));
-			if (real_errno != EPERM) 
-				return false;
+			pwd = getpwuid(Conf_UID);
+			Log(LOG_ERR, "Can't change user ID to %s(%u): %s",
+			    pwd ? pwd->pw_name : "?", Conf_UID,
+			    strerror(errno));
+			if (real_errno != EPERM)
+				goto out;
 		}
 	}
 
@@ -757,65 +700,82 @@ NGIRCd_Init( bool NGIRCd_NoDaemon )
 	/* Normally a child process is forked which isn't any longer
 	 * connected to ther controlling terminal. Use "--nodaemon"
 	 * to disable this "daemon mode" (useful for debugging). */
-	if ( ! NGIRCd_NoDaemon ) {
-		pid = fork( );
-		if( pid > 0 ) {
+	if (!NGIRCd_NoDaemon) {
+		pid = fork();
+		if (pid > 0) {
 			/* "Old" process: exit. */
-			exit( 0 );
+			exit(0);
 		}
-		if( pid < 0 ) {
+		if (pid < 0) {
 			/* Error!? */
-			fprintf( stderr, "%s: Can't fork: %s!\nFatal error, exiting now ...\n",
-								PACKAGE_NAME, strerror( errno ));
-			exit( 1 );
+			fprintf(stderr,
+				"%s: Can't fork: %s!\nFatal error, exiting now ...\n",
+				PACKAGE_NAME, strerror(errno));
+			exit(1);
 		}
 
 		/* New child process */
-		(void)setsid( );
-		chdir( "/" );
+#ifndef NeXT
+		(void)setsid();
+#else
+		setpgrp(0, getpid());
+#endif
+		if (chdir("/") != 0)
+			Log(LOG_ERR, "Can't change directory to '/': %s",
+				     strerror(errno));
 
 		/* Detach stdin, stdout and stderr */
-		Setup_FDStreams( );
+		Setup_FDStreams(fd);
+		if (fd > 2)
+			close(fd);
 	}
 	pid = getpid();
 
-	Pidfile_Create( pid );
+	Pidfile_Create(pid);
 
 	/* Check UID/GID we are running as, can be different from values
 	 * configured (e. g. if we were already started with a UID>0. */
 	Conf_UID = getuid();
 	Conf_GID = getgid();
 
-	pwd = getpwuid( Conf_UID );
-	grp = getgrgid( Conf_GID );
+	pwd = getpwuid(Conf_UID);
+	grp = getgrgid(Conf_GID);
 
-	Log( LOG_INFO, "Running as user %s(%ld), group %s(%ld), with PID %ld.",
-				pwd ? pwd->pw_name : "unknown", Conf_UID,
-				grp ? grp->gr_name : "unknown", Conf_GID, pid);
+	Log(LOG_INFO, "Running as user %s(%ld), group %s(%ld), with PID %ld.",
+	    pwd ? pwd->pw_name : "unknown", (long)Conf_UID,
+	    grp ? grp->gr_name : "unknown", (long)Conf_GID, (long)pid);
 
-	if ( chrooted ) {
-		Log( LOG_INFO, "Running chrooted, chrootdir \"%s\".",  Conf_Chroot );
+	if (chrooted) {
+		Log(LOG_INFO, "Running with root directory \"%s\".",
+		    Conf_Chroot );
 		return true;
-	} else {
-		Log( LOG_INFO, "Not running chrooted." );
-	}
+	} else
+		Log(LOG_INFO, "Not running with changed root directory.");
 
-	/* Change working directory to home directory of the user
-	 * we are running as (only when running in daemon mode and not in chroot) */
-	
-	if ( pwd ) {
-		if (!NGIRCd_NoDaemon ) {
-			if( chdir( pwd->pw_dir ) == 0 ) 
-				Log( LOG_DEBUG, "Changed working directory to \"%s\" ...", pwd->pw_dir );
-			else 
-				Log( LOG_INFO, "Notice: Can't change working directory to \"%s\": %s",
-								pwd->pw_dir, strerror( errno ));
-		}
-	} else {
-		Log( LOG_ERR, "Can't get user informaton for UID %d!?", Conf_UID );
-	}
+	/* Change working directory to home directory of the user we are
+	 * running as (only when running in daemon mode and not in chroot) */
+
+	if (NGIRCd_NoDaemon)
+		return true;
+
+	if (pwd) {
+		if (chdir(pwd->pw_dir) == 0)
+			Log(LOG_DEBUG,
+			    "Changed working directory to \"%s\" ...",
+			    pwd->pw_dir);
+		else
+			Log(LOG_INFO,
+			    "Notice: Can't change working directory to \"%s\": %s",
+			    pwd->pw_dir, strerror(errno));
+	} else
+		Log(LOG_ERR, "Can't get user informaton for UID %d!?", Conf_UID);
+
+	return true;
+ out:
+	if (fd > 2)
+		close(fd);
+	return false;
+} /* NGIRCd_Init */
 
-return true;
-}
 
 /* -eof- */
