@@ -1,20 +1,22 @@
 /*
  * ngIRCd -- The Next Generation IRC Daemon
- * Copyright (c)2001-2003 by Alexander Barton (alex@barton.de)
+ * Copyright (c)2001-2011 Alexander Barton (alex@barton.de) and Contributors.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  * Please read the file COPYING, README and AUTHORS for more information.
- *
- * Asynchronous resolver
  */
 
+#define RESOLVER_TIMEOUT (Conf_PongTimeout*3)/4
 
 #include "portab.h"
 
-static char UNUSED id[] = "$Id: resolve.c,v 1.29 2008/02/26 22:04:17 fw Exp $";
+/**
+ * @file
+ * Asynchronous resolver
+ */
 
 #include "imp.h"
 #include <assert.h>
@@ -25,6 +27,7 @@ static char UNUSED id[] = "$Id: resolve.c,v 1.29 2008/02/26 22:04:17 fw Exp $";
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netdb.h>
+#include <signal.h>
 
 #ifdef IDENTAUTH
 #ifdef HAVE_IDENT_H
@@ -32,9 +35,12 @@ static char UNUSED id[] = "$Id: resolve.c,v 1.29 2008/02/26 22:04:17 fw Exp $";
 #endif
 #endif
 
+#include "array.h"
 #include "conn.h"
+#include "conf.h"
 #include "defines.h"
 #include "log.h"
+#include "ng_ipaddr.h"
 
 #include "exp.h"
 #include "resolve.h"
@@ -43,46 +49,18 @@ static char UNUSED id[] = "$Id: resolve.c,v 1.29 2008/02/26 22:04:17 fw Exp $";
 
 static void Do_ResolveAddr PARAMS(( const ng_ipaddr_t *Addr, int Sock, int w_fd ));
 static void Do_ResolveName PARAMS(( const char *Host, int w_fd ));
-static bool register_callback PARAMS((RES_STAT *s, void (*cbfunc)(int, short)));
 
 #ifdef WANT_IPV6
 extern bool Conf_ConnectIPv4;
 extern bool Conf_ConnectIPv6;
 #endif
 
-static pid_t
-Resolver_fork(int *pipefds)
-{
-	pid_t pid;
-
-	if (pipe(pipefds) != 0) {
-                Log( LOG_ALERT, "Resolver: Can't create output pipe: %s!", strerror( errno ));
-                return -1;
-	}
-
-	pid = fork();
-	switch(pid) {
-		case -1:
-			Log( LOG_CRIT, "Resolver: Can't fork: %s!", strerror( errno ));
-			close(pipefds[0]);
-			close(pipefds[1]);
-			return -1;
-		case 0: /* child */
-			close(pipefds[0]);
-			Log_Init_Resolver( );
-			return 0;
-	}
-	/* parent */
-	close(pipefds[1]);
-	return pid; 
-}
-
 
 /**
  * Resolve IP (asynchronous!).
  */
 GLOBAL bool
-Resolve_Addr(RES_STAT * s, const ng_ipaddr_t *Addr, int identsock,
+Resolve_Addr(PROC_STAT * s, const ng_ipaddr_t *Addr, int identsock,
 	     void (*cbfunc) (int, short))
 {
 	int pipefd[2];
@@ -90,17 +68,16 @@ Resolve_Addr(RES_STAT * s, const ng_ipaddr_t *Addr, int identsock,
 
 	assert(s != NULL);
 
-	pid = Resolver_fork(pipefd);
+	pid = Proc_Fork(s, pipefd, cbfunc, RESOLVER_TIMEOUT);
 	if (pid > 0) {
-		Log(LOG_DEBUG, "Resolver for %s created (PID %d).", ng_ipaddr_tostr(Addr), pid);
-
-		s->pid = pid;
-		s->resolver_fd = pipefd[0];
-		return register_callback(s, cbfunc);
+		LogDebug("Resolver for %s created (PID %d).", ng_ipaddr_tostr(Addr), pid);
+		return true;
 	} else if( pid == 0 ) {
 		/* Sub process */
-		Do_ResolveAddr( Addr, identsock, pipefd[1]);
-		Log_Exit_Resolver( );
+		Log_Init_Subprocess("Resolver");
+		Conn_CloseAllSockets(identsock);
+		Do_ResolveAddr(Addr, identsock, pipefd[1]);
+		Log_Exit_Subprocess("Resolver");
 		exit(0);
 	}
 	return false;
@@ -111,43 +88,34 @@ Resolve_Addr(RES_STAT * s, const ng_ipaddr_t *Addr, int identsock,
  * Resolve hostname (asynchronous!).
  */
 GLOBAL bool
-Resolve_Name( RES_STAT *s, const char *Host, void (*cbfunc)(int, short))
+Resolve_Name( PROC_STAT *s, const char *Host, void (*cbfunc)(int, short))
 {
 	int pipefd[2];
 	pid_t pid;
 
 	assert(s != NULL);
 
-	pid = Resolver_fork(pipefd);
+	pid = Proc_Fork(s, pipefd, cbfunc, RESOLVER_TIMEOUT);
 	if (pid > 0) {
 		/* Main process */
 #ifdef DEBUG
 		Log( LOG_DEBUG, "Resolver for \"%s\" created (PID %d).", Host, pid );
 #endif
-		s->pid = pid;
-		s->resolver_fd = pipefd[0];
-		return register_callback(s, cbfunc);
+		return true;
 	} else if( pid == 0 ) {
 		/* Sub process */
+		Log_Init_Subprocess("Resolver");
+		Conn_CloseAllSockets(NONE);
 		Do_ResolveName(Host, pipefd[1]);
-		Log_Exit_Resolver( );
+		Log_Exit_Subprocess("Resolver");
 		exit(0);
 	}
 	return false;
 } /* Resolve_Name */
 
 
-GLOBAL void
-Resolve_Init(RES_STAT *s)
-{
-	assert(s != NULL);
-	s->resolver_fd = -1;
-	s->pid = 0;
-}
-
-
-#ifndef WANT_IPV6
-#ifdef h_errno
+#if !defined(HAVE_GETADDRINFO) || !defined(HAVE_GETNAMEINFO)
+#if !defined(WANT_IPV6) && defined(h_errno)
 static char *
 Get_Error( int H_Error )
 {
@@ -164,8 +132,8 @@ Get_Error( int H_Error )
 	}
 	return "unknown error";
 }
-#endif /* h_errno */
-#endif /* WANT_IPV6 */
+#endif
+#endif
 
 
 /* Do "IDENT" (aka "AUTH") lookup and append result to resolved_addr array */
@@ -175,22 +143,24 @@ Do_IdentQuery(int identsock, array *resolved_addr)
 #ifdef IDENTAUTH
 	char *res;
 
-	assert(identsock >= 0);
-
-#ifdef DEBUG
-	Log_Resolver(LOG_DEBUG, "Doing IDENT lookup on socket %d ...", identsock);
-#endif
 	if (identsock < 0)
 		return;
+
+#ifdef DEBUG
+	Log_Subprocess(LOG_DEBUG, "Doing IDENT lookup on socket %d ...",
+		       identsock);
+#endif
 	res = ident_id( identsock, 10 );
 #ifdef DEBUG
-	Log_Resolver(LOG_DEBUG, "Ok, IDENT lookup on socket %d done: \"%s\"",
-						identsock, res ? res : "(NULL)" );
+	Log_Subprocess(LOG_DEBUG, "Ok, IDENT lookup on socket %d done: \"%s\"",
+		       identsock, res ? res : "(NULL)");
 #endif
 	if (!res) /* no result */
 		return;
 	if (!array_cats(resolved_addr, res))
-		Log_Resolver(LOG_WARNING, "Resolver: Cannot copy IDENT result: %s!", strerror(errno));
+		Log_Subprocess(LOG_WARNING,
+			       "Resolver: Cannot copy IDENT result: %s!",
+			       strerror(errno));
 
 	free(res);
 #else
@@ -206,7 +176,7 @@ Do_IdentQuery(int identsock, array *resolved_addr)
  * the IP address in resbuf and returns false.
  * @param IpAddr ip address to resolve
  * @param resbuf result buffer to store DNS name/string representation of ip address
- * @reslen size of result buffer (must be >= NGT_INET_ADDRSTRLEN)
+ * @param reslen size of result buffer (must be >= NGT_INET_ADDRSTRLEN)
  * @return true if reverse lookup successful, false otherwise
  */
 static bool
@@ -221,7 +191,7 @@ ReverseLookup(const ng_ipaddr_t *IpAddr, char *resbuf, size_t reslen)
 	*resbuf = 0;
 
 	res = getnameinfo((struct sockaddr *) IpAddr, ng_ipaddr_salen(IpAddr),
-				resbuf, reslen, NULL, 0, NI_NAMEREQD);
+			  resbuf, (socklen_t)reslen, NULL, 0, NI_NAMEREQD);
 	if (res == 0)
 		return true;
 
@@ -252,7 +222,7 @@ ReverseLookup(const ng_ipaddr_t *IpAddr, char *resbuf, size_t reslen)
 	assert(reslen >= NG_INET_ADDRSTRLEN);
 	ng_ipaddr_tostr_r(IpAddr, tmp_ip_str);
 
-	Log_Resolver(LOG_WARNING, "%s: Can't resolve address \"%s\": %s",
+	Log_Subprocess(LOG_WARNING, "%s: Can't resolve address \"%s\": %s",
 				funcname, tmp_ip_str, errmsg);
 	strlcpy(resbuf, tmp_ip_str, reslen);
 	return false;
@@ -268,38 +238,32 @@ ReverseLookup(const ng_ipaddr_t *IpAddr, char *resbuf, size_t reslen)
  * @return true if lookup successful, false if domain name not found
  */
 static bool
-ForwardLookup(const char *hostname, array *IpAddr)
+ForwardLookup(const char *hostname, array *IpAddr, int af)
 {
 	ng_ipaddr_t addr;
+
 #ifdef HAVE_GETADDRINFO
 	int res;
 	struct addrinfo *a, *ai_results;
-	static struct addrinfo hints = {
-#ifndef WANT_IPV6
-		.ai_family = AF_INET,
-#endif
-#ifdef AI_ADDRCONFIG	/* glibc has this, but not e.g. netbsd 4.0 */
-		.ai_flags = AI_ADDRCONFIG,
-#endif
-		.ai_socktype = SOCK_STREAM,
-		.ai_protocol = IPPROTO_TCP
-	};
-#ifdef WANT_IPV6
-	assert(Conf_ConnectIPv6 || Conf_ConnectIPv4);
+	static struct addrinfo hints;
 
-	if (!Conf_ConnectIPv6)
-		hints.ai_family = AF_INET;
-	if (!Conf_ConnectIPv4)
-		hints.ai_family = AF_INET6;
+#ifdef AI_ADDRCONFIG	/* glibc has this, but not e.g. netbsd 4.0 */
+	hints.ai_flags = AI_ADDRCONFIG;
 #endif
+	hints.ai_socktype = SOCK_STREAM;
+	hints.ai_protocol = IPPROTO_TCP;
+	hints.ai_family = af;
+
+	memset(&addr, 0, sizeof(addr));
+
 	res = getaddrinfo(hostname, NULL, &hints, &ai_results);
 	switch (res) {
 	case 0:	break;
 	case EAI_SYSTEM:
-		Log_Resolver(LOG_WARNING, "Can't resolve \"%s\": %s", hostname, strerror(errno));
+		Log_Subprocess(LOG_WARNING, "Can't resolve \"%s\": %s", hostname, strerror(errno));
 		return false;
 	default:
-		Log_Resolver(LOG_WARNING, "Can't resolve \"%s\": %s", hostname, gai_strerror(res));
+		Log_Subprocess(LOG_WARNING, "Can't resolve \"%s\": %s", hostname, gai_strerror(res));
 		return false;
 	}
 
@@ -322,9 +286,10 @@ ForwardLookup(const char *hostname, array *IpAddr)
 
 	if (!h) {
 #ifdef h_errno
-		Log_Resolver(LOG_WARNING, "Can't resolve \"%s\": %s", hostname, Get_Error(h_errno));
+		Log_Subprocess(LOG_WARNING, "Can't resolve \"%s\": %s",
+			       hostname, Get_Error(h_errno));
 #else
-		Log_Resolver(LOG_WARNING, "Can't resolve \"%s\"", hostname);
+		Log_Subprocess(LOG_WARNING, "Can't resolve \"%s\"", hostname);
 #endif
 		return false;
 	}
@@ -360,7 +325,7 @@ Addr_in_list(const array *resolved_addr, const ng_ipaddr_t *Addr)
 	tmpAddrs = array_start(resolved_addr);
 
 	while (len > 0) {
-		Log_Resolver(LOG_WARNING, "Address mismatch: %s != %s",
+		Log_Subprocess(LOG_WARNING, "Address mismatch: %s != %s",
 			tmp_ip_str, ng_ipaddr_tostr(tmpAddrs));
 		tmpAddrs++;
 		len--;
@@ -373,15 +338,15 @@ Addr_in_list(const array *resolved_addr, const ng_ipaddr_t *Addr)
 static void
 Log_Forgery_NoIP(const char *ip, const char *host)
 {
-	Log_Resolver(LOG_WARNING, "Possible forgery: %s resolved to %s "
-		"(which has no ip address)", ip, host);
+	Log_Subprocess(LOG_WARNING,
+		"Possible forgery: %s resolved to %s (which has no ip address)", ip, host);
 }
 
 static void
 Log_Forgery_WrongIP(const char *ip, const char *host)
 {
-	Log_Resolver(LOG_WARNING,"Possible forgery: %s resolved to %s "
-		"(which points to different address)", ip, host);
+	Log_Subprocess(LOG_WARNING,
+		"Possible forgery: %s resolved to %s (which points to different address)", ip, host);
 }
 
 
@@ -394,7 +359,7 @@ ArrayWrite(int fd, const array *a)
 	assert(data);
 
 	if( (size_t)write(fd, data, len) != len )
-		Log_Resolver( LOG_CRIT, "Resolver: Can't write to parent: %s!",
+		Log_Subprocess( LOG_CRIT, "Resolver: Can't write to parent: %s!",
 							strerror(errno));
 }
 
@@ -412,12 +377,12 @@ Do_ResolveAddr(const ng_ipaddr_t *Addr, int identsock, int w_fd)
 	array_init(&resolved_addr);
 	ng_ipaddr_tostr_r(Addr, tmp_ip_str);
 #ifdef DEBUG
-	Log_Resolver(LOG_DEBUG, "Now resolving %s ...", tmp_ip_str);
+	Log_Subprocess(LOG_DEBUG, "Now resolving %s ...", tmp_ip_str);
 #endif
 	if (!ReverseLookup(Addr, hostname, sizeof(hostname)))
 		goto dns_done;
 
-	if (ForwardLookup(hostname, &resolved_addr)) {
+	if (ForwardLookup(hostname, &resolved_addr, ng_ipaddr_af(Addr))) {
 		if (!Addr_in_list(&resolved_addr, Addr)) {
 			Log_Forgery_WrongIP(tmp_ip_str, hostname);
 			strlcpy(hostname, tmp_ip_str, sizeof(hostname));
@@ -427,13 +392,15 @@ Do_ResolveAddr(const ng_ipaddr_t *Addr, int identsock, int w_fd)
 		strlcpy(hostname, tmp_ip_str, sizeof(hostname));
 	}
 #ifdef DEBUG
-	Log_Resolver(LOG_DEBUG, "Ok, translated %s to \"%s\".", tmp_ip_str, hostname);
+	Log_Subprocess(LOG_DEBUG, "Ok, translated %s to \"%s\".", tmp_ip_str, hostname);
 #endif
  dns_done:
 	len = strlen(hostname);
 	hostname[len] = '\n';
 	if (!array_copyb(&resolved_addr, hostname, ++len)) {
-		Log_Resolver(LOG_CRIT, "Resolver: Can't copy resolved name: %s!", strerror(errno));
+		Log_Subprocess(LOG_CRIT,
+			       "Resolver: Can't copy resolved name: %s!",
+			       strerror(errno));
 		array_free(&resolved_addr);
 		return;
 	}
@@ -452,15 +419,27 @@ Do_ResolveName( const char *Host, int w_fd )
 	/* Resolver sub-process: resolve name and write result into pipe
 	 * to parent. */
 	array IpAddrs;
+	int af;
 #ifdef DEBUG
 	ng_ipaddr_t *addr;
 	size_t len;
 #endif
-	Log_Resolver(LOG_DEBUG, "Now resolving \"%s\" ...", Host);
+	Log_Subprocess(LOG_DEBUG, "Now resolving \"%s\" ...", Host);
 
 	array_init(&IpAddrs);
-	/* Resolve hostname */
-	if (!ForwardLookup(Host, &IpAddrs)) {
+
+#ifdef WANT_IPV6
+	af = AF_UNSPEC;
+	assert(Conf_ConnectIPv6 || Conf_ConnectIPv4);
+
+	if (!Conf_ConnectIPv6)
+		af = AF_INET;
+	if (!Conf_ConnectIPv4)
+		af = AF_INET6;
+#else
+	af = AF_INET;
+#endif
+	if (!ForwardLookup(Host, &IpAddrs, af)) {
 		close(w_fd);
 		return;
 	}
@@ -470,7 +449,7 @@ Do_ResolveName( const char *Host, int w_fd )
 	addr = array_start(&IpAddrs);
 	assert(addr);
 	for (; len > 0; --len,addr++) {
-		Log_Resolver(LOG_DEBUG, "translated \"%s\" to %s.",
+		Log_Subprocess(LOG_DEBUG, "translated \"%s\" to %s.",
 					Host, ng_ipaddr_tostr(addr));
 	}
 #endif
@@ -481,65 +460,4 @@ Do_ResolveName( const char *Host, int w_fd )
 } /* Do_ResolveName */
 
 
-static bool
-register_callback( RES_STAT *s, void (*cbfunc)(int, short))
-{
-	assert(cbfunc != NULL);
-	assert(s != NULL);
-	assert(s->resolver_fd >= 0);
-
-	if (io_setnonblock(s->resolver_fd) &&
-		io_event_create(s->resolver_fd, IO_WANTREAD, cbfunc))
-			return true;
-
-	Log( LOG_CRIT, "Resolver: Could not register callback function: %s!", strerror(errno));
-	close(s->resolver_fd);
-	Resolve_Init(s);
-	return false;
-}
-
-
-GLOBAL bool
-Resolve_Shutdown( RES_STAT *s)
-{
-	bool ret = false;
-
-	assert(s != NULL);
-	assert(s->resolver_fd >= 0);
-
-	if (s->resolver_fd >= 0)
-		ret = io_close(s->resolver_fd);
-
-	Resolve_Init(s);
-	return ret;
-}
-
-
-/**
- * Read result of resolver sub-process from pipe
- */
-GLOBAL size_t
-Resolve_Read( RES_STAT *s, void* readbuf, size_t buflen)
-{
-	ssize_t bytes_read;
-
-	assert(buflen > 0);
-
-	/* Read result from pipe */
-	bytes_read = read(s->resolver_fd, readbuf, buflen);
-	if (bytes_read < 0) {
-		if (errno == EAGAIN)
-			return 0;
-
-		Log( LOG_CRIT, "Resolver: Can't read result: %s!", strerror(errno));
-		bytes_read = 0;
-	}
-#ifdef DEBUG
-	else if (bytes_read == 0)
-		Log( LOG_DEBUG, "Resolver: Can't read result: EOF");
-#endif
-	Resolve_Shutdown(s);
-	return (size_t)bytes_read;
-}
 /* -eof- */
-
